#    Copyright 2017 AT&T Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import

import functools
import mock
from oslo_config import cfg

import fixtures
from tempest.lib import exceptions

from patrole_tempest_plugin import rbac_exceptions
from patrole_tempest_plugin import rbac_rule_validation as rbac_rv
from patrole_tempest_plugin.tests.unit import base
from patrole_tempest_plugin.tests.unit import fixtures as patrole_fixtures

CONF = cfg.CONF


class BaseRBACRuleValidationTest(base.TestCase):

    test_roles = ['member']

    def setUp(self):
        super(BaseRBACRuleValidationTest, self).setUp()
        self.rbac_utils_fixture = self.useFixture(
            patrole_fixtures.RbacUtilsMixinFixture(
                rbac_test_roles=self.test_roles))
        self.test_obj = self.rbac_utils_fixture.test_obj


class BaseRBACMultiRoleRuleValidationTest(BaseRBACRuleValidationTest):

    test_roles = ['member', 'anotherrole']


class RBACRuleValidationTest(BaseRBACRuleValidationTest):
    """Test suite for validating fundamental functionality for the
    ``rbac_rule_validation`` decorator.
    """

    def setUp(self):
        super(RBACRuleValidationTest, self).setUp()
        # This behavior is tested in separate test class below.
        self.useFixture(fixtures.MockPatchObject(
            rbac_rv, '_validate_override_role_called'))

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_have_permission_no_exc(self, mock_authority,
                                                    mock_log):
        """Test that having permission and no exception thrown is success.

        Positive test case success scenario.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value = True

        @rbac_rv.action(mock.sentinel.service, rules=[mock.sentinel.action])
        def test_policy(*args):
            pass

        test_policy(self.test_obj)
        mock_log.error.assert_not_called()

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_lack_permission_throw_exc(self, mock_authority,
                                                       mock_log):
        """Test that having no permission and exception thrown is success.

        Negative test case success scenario.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value =\
            False

        @rbac_rv.action(mock.sentinel.service, rules=[mock.sentinel.action])
        def test_policy(*args):
            raise exceptions.Forbidden()

        test_policy(self.test_obj)
        mock_log.error.assert_not_called()

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_forbidden_negative(self, mock_authority,
                                                mock_log):
        """Test RbacUnderPermissionException error is thrown and have
        permission fails.

        Negative test case: if Forbidden is thrown and the user should be
        allowed to perform the action, then the RbacUnderPermissionException
        exception should be raised.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value = True

        @rbac_rv.action(mock.sentinel.service, rules=[mock.sentinel.action])
        def test_policy(*args):
            raise exceptions.Forbidden()

        test_re = ("User with roles \['member'\] was not allowed to perform "
                   "the following actions: \[%s\].*" % (mock.sentinel.action))
        self.assertRaisesRegex(
            rbac_exceptions.RbacUnderPermissionException, test_re, test_policy,
            self.test_obj)
        self.assertRegex(mock_log.error.mock_calls[0][1][0], test_re)

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_rbac_failed_response_body_positive(
            self, mock_authority, mock_log):
        """Test BasePatroleResponseBodyException error is thrown without
        permission passes.

        Positive test case: if subclass of BasePatroleResponseBodyException is
        thrown and the user is not allowed to perform the action, then this is
        a success.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value =\
            False

        def _do_test(exception_cls, **kwargs):
            @rbac_rv.action(mock.sentinel.service,
                            rules=[mock.sentinel.action])
            def test_policy(*args):
                raise exception_cls(**kwargs)

            mock_log.error.assert_not_called()
            mock_log.warning.assert_not_called()

        _do_test(rbac_exceptions.RbacMissingAttributeResponseBody,
                 attribute=mock.sentinel.attr)
        _do_test(rbac_exceptions.RbacPartialResponseBody,
                 body=mock.sentinel.body)
        _do_test(rbac_exceptions.RbacEmptyResponseBody)

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_soft_authorization_exceptions(
            self, mock_authority, mock_log):
        """Test RbacUnderPermissionException error is thrown when any of the
        soft authorization-related exceptions are raised by a test.

        Negative test case: if subclass of BasePatroleResponseBodyException is
        thrown and the user is allowed to perform the action, then this is an
        expected failure.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value = True

        def _do_test(exception_cls, **kwargs):
            @rbac_rv.action(mock.sentinel.service,
                            rules=[mock.sentinel.action])
            def test_policy(*args):
                raise exception_cls(**kwargs)

            test_re = (".*User with roles \[%s\] was not allowed to "
                       "perform the following actions: \[%s\].*" % (
                           ', '.join("'%s'" % r for r in self.test_roles),
                           mock.sentinel.action))
            self.assertRaisesRegex(
                rbac_exceptions.RbacUnderPermissionException, test_re,
                test_policy, self.test_obj)
            self.assertRegex(mock_log.error.mock_calls[0][1][0], test_re)

        _do_test(rbac_exceptions.RbacMissingAttributeResponseBody,
                 attribute=mock.sentinel.attr)
        _do_test(rbac_exceptions.RbacPartialResponseBody,
                 body=mock.sentinel.body)
        _do_test(rbac_exceptions.RbacEmptyResponseBody)

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_expect_not_found_but_raises_forbidden(self, mock_authority,
                                                   mock_log):
        """Test that expecting 404 but getting 403 works for all scenarios.

        Tests the following scenarios:
        1) Test no permission and 404 is expected but 403 is thrown throws
           exception.
        2) Test have permission and 404 is expected but 403 is thrown throws
           exception.
        """
        @rbac_rv.action(mock.sentinel.service, rules=[mock.sentinel.action],
                        expected_error_codes=[404])
        def test_policy(*args):
            raise exceptions.Forbidden('Test message')

        error_re = r'Expected .* to be raised but .* was raised instead'

        for allowed in [True, False]:
            mock_authority.PolicyAuthority.return_value.allowed.\
                return_value = allowed
            self.assertRaisesRegex(
                rbac_exceptions.RbacExpectedWrongException, error_re,
                test_policy, self.test_obj)
            self.assertTrue(mock_log.error.called)

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_expect_not_found_and_raise_not_found(self, mock_authority,
                                                  mock_log):
        """Test that expecting 404 and getting 404 works for all scenarios.

        Tests the following scenarios:
        1) Test no permission and 404 is expected and 404 is thrown succeeds.
        2) Test have permission and 404 is expected and 404 is thrown fails.

        In both cases, a LOG.warning is called with the "irregular message"
        that signals to user that a 404 was expected and caught.
        """
        policy_names = ['foo:bar']

        @rbac_rv.action(mock.sentinel.service, rules=policy_names,
                        expected_error_codes=[404])
        def test_policy(*args):
            raise exceptions.NotFound()

        expected_errors = [
            ("User with roles \['member'\] was not allowed to perform the "
             "following actions: \['%s'\].*" % policy_names[0]),
            None
        ]

        for pos, allowed in enumerate([True, False]):
            mock_authority.PolicyAuthority.return_value.allowed\
                .return_value = allowed

            error_re = expected_errors[pos]

            if error_re:
                self.assertRaisesRegex(
                    rbac_exceptions.RbacUnderPermissionException, error_re,
                    test_policy, self.test_obj)
                self.assertRegex(mock_log.error.mock_calls[0][1][0], error_re)
            else:
                test_policy(self.test_obj)
                mock_log.error.assert_not_called()

            mock_log.warning.assert_called_with(
                "NotFound exception was caught for test %s. Expected policies "
                "which may have caused the error: %s. The service %s throws a "
                "404 instead of a 403, which is irregular",
                test_policy.__name__,
                ', '.join(policy_names),
                mock.sentinel.service)

            mock_log.warning.reset_mock()
            mock_log.error.reset_mock()

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_overpermission_negative(self, mock_authority,
                                                     mock_log):
        """Test that RbacOverPermissionException is correctly handled.

        Tests that case where no exception is thrown but the Patrole framework
        says that the role should not be allowed to perform the policy action.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value =\
            False

        @rbac_rv.action(mock.sentinel.service, rules=[mock.sentinel.action])
        def test_policy_expect_forbidden(*args):
            pass

        @rbac_rv.action(mock.sentinel.service, rules=[mock.sentinel.action],
                        expected_error_codes=[404])
        def test_policy_expect_not_found(*args):
            pass

        for test_policy in (
            test_policy_expect_forbidden, test_policy_expect_not_found):

            error_re = ".*OverPermission: .* \[%s\]$" % mock.sentinel.action
            self.assertRaisesRegex(rbac_exceptions.RbacOverPermissionException,
                                   error_re, test_policy, self.test_obj)
            self.assertRegex(mock_log.error.mock_calls[0][1][0], error_re)
            mock_log.error.reset_mock()

    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_invalid_policy_rule_raises_parsing_exception(
            self, mock_authority):
        """Test that invalid policy action causes test to raise an exception.
        """
        mock_authority.PolicyAuthority.return_value.allowed.\
            side_effect = rbac_exceptions.RbacParsingException

        @rbac_rv.action(mock.sentinel.service, mock.sentinel.action)
        def test_policy(*args):
            pass

        error_re = 'Attempted to test an invalid policy file or action'
        self.assertRaisesRegex(rbac_exceptions.RbacParsingException, error_re,
                               test_policy, self.test_obj)

        mock_authority.PolicyAuthority.assert_called_once_with(
            mock.sentinel.project_id, mock.sentinel.user_id,
            mock.sentinel.service, extra_target_data={})

    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_get_exception_type_404(self, _):
        """Test that getting a 404 exception type returns NotFound."""
        expected_exception = exceptions.NotFound
        expected_irregular_msg = (
            "NotFound exception was caught for test %s. Expected policies "
            "which may have caused the error: %s. The service %s throws a "
            "404 instead of a 403, which is irregular")

        actual_exception, actual_irregular_msg = \
            rbac_rv._get_exception_type(404)

        self.assertEqual(expected_exception, actual_exception)
        self.assertEqual(expected_irregular_msg, actual_irregular_msg)

    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_get_exception_type_403(self, _):
        """Test that getting a 403 exception type returns Forbidden."""
        expected_exception = exceptions.Forbidden
        expected_irregular_msg = None

        actual_exception, actual_irregular_msg = \
            rbac_rv._get_exception_type(403)

        self.assertEqual(expected_exception, actual_exception)
        self.assertEqual(expected_irregular_msg, actual_irregular_msg)

    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    def test_exception_thrown_when_type_is_not_int(self, mock_log, _):
        """Test that non-integer exception type raises error."""
        self.assertRaises(rbac_exceptions.RbacInvalidErrorCode,
                          rbac_rv._get_exception_type, "403")

        mock_log.error.assert_called_once_with("Please pass an expected error "
                                               "code. Currently supported "
                                               "codes: [403, 404]")

    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    def test_exception_thrown_when_type_is_403_or_404(self, mock_log, _):
        """Test that unsupported exceptions throw error."""
        invalid_exceptions = [200, 400, 500]
        for exc in invalid_exceptions:
            self.assertRaises(rbac_exceptions.RbacInvalidErrorCode,
                              rbac_rv._get_exception_type, exc)
            mock_log.error.assert_called_once_with(
                "Please pass an expected error code. Currently supported "
                "codes: [403, 404]")

            mock_log.error.reset_mock()


class RBACMultiRoleRuleValidationTest(BaseRBACMultiRoleRuleValidationTest,
                                      RBACRuleValidationTest):
    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_forbidden_negative(self, mock_authority,
                                                mock_log):
        """Test RbacUnderPermissionException error is thrown and have
        permission fails.

        Negative test case: if Forbidden is thrown and the user should be
        allowed to perform the action, then the RbacUnderPermissionException
        exception should be raised.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value = True

        @rbac_rv.action(mock.sentinel.service, rules=[mock.sentinel.action])
        def test_policy(*args):
            raise exceptions.Forbidden()

        test_re = ("User with roles \['member', 'anotherrole'\] was not "
                   "allowed to perform the following actions: \[%s\].*" %
                   (mock.sentinel.action))
        self.assertRaisesRegex(
            rbac_exceptions.RbacUnderPermissionException, test_re, test_policy,
            self.test_obj)
        self.assertRegex(mock_log.error.mock_calls[0][1][0], test_re)

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_expect_not_found_and_raise_not_found(self, mock_authority,
                                                  mock_log):
        """Test that expecting 404 and getting 404 works for all scenarios.

        Tests the following scenarios:
        1) Test no permission and 404 is expected and 404 is thrown succeeds.
        2) Test have permission and 404 is expected and 404 is thrown fails.

        In both cases, a LOG.warning is called with the "irregular message"
        that signals to user that a 404 was expected and caught.
        """
        policy_names = ['foo:bar']

        @rbac_rv.action(mock.sentinel.service, rules=policy_names,
                        expected_error_codes=[404])
        def test_policy(*args):
            raise exceptions.NotFound()

        expected_errors = [
            ("User with roles \['member', 'anotherrole'\] was not allowed to "
             "perform the following actions: \['%s'\].*" % policy_names[0]),
            None
        ]

        for pos, allowed in enumerate([True, False]):
            mock_authority.PolicyAuthority.return_value.allowed\
                .return_value = allowed

            error_re = expected_errors[pos]

            if error_re:
                self.assertRaisesRegex(
                    rbac_exceptions.RbacUnderPermissionException, error_re,
                    test_policy, self.test_obj)
                self.assertRegex(mock_log.error.mock_calls[0][1][0], error_re)
            else:
                test_policy(self.test_obj)
                mock_log.error.assert_not_called()

            mock_log.warning.assert_called_with(
                "NotFound exception was caught for test %s. Expected policies "
                "which may have caused the error: %s. The service %s throws a "
                "404 instead of a 403, which is irregular",
                test_policy.__name__,
                ', '.join(policy_names),
                mock.sentinel.service)

            mock_log.warning.reset_mock()
            mock_log.error.reset_mock()


class RBACRuleValidationLoggingTest(BaseRBACRuleValidationTest):
    """Test class for validating the RBAC log, dedicated to just logging
    Patrole RBAC validation work flows.
    """

    def setUp(self):
        super(RBACRuleValidationLoggingTest, self).setUp()
        # This behavior is tested in separate test class below.
        self.useFixture(fixtures.MockPatchObject(
            rbac_rv, '_validate_override_role_called'))

    @mock.patch.object(rbac_rv, 'RBACLOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rbac_report_logging_disabled(self, mock_authority, mock_rbaclog):
        """Test case to ensure that we DON'T write logs when  enable_reporting
        is False
        """
        self.useFixture(
            patrole_fixtures.ConfPatcher(enable_reporting=False,
                                         group='patrole_log'))

        mock_authority.PolicyAuthority.return_value.allowed.return_value = True

        @rbac_rv.action(mock.sentinel.service, rules=[mock.sentinel.action])
        def test_policy(*args):
            pass

        test_policy(self.test_obj)
        self.assertFalse(mock_rbaclog.info.called)

    @mock.patch.object(rbac_rv, 'RBACLOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rbac_report_logging_enabled(self, mock_authority, mock_rbaclog):
        """Test case to ensure that we DO write logs when enable_reporting is
        True
        """
        self.useFixture(
            patrole_fixtures.ConfPatcher(enable_reporting=True,
                                         group='patrole_log'))

        mock_authority.PolicyAuthority.return_value.allowed.return_value = True
        policy_names = ['foo:bar', 'baz:qux']

        @rbac_rv.action(mock.sentinel.service, rules=policy_names)
        def test_policy(*args):
            pass

        test_policy(self.test_obj)
        mock_rbaclog.info.assert_called_once_with(
            "[Service]: %s, [Test]: %s, [Rules]: %s, "
            "[Expected]: %s, [Actual]: %s",
            mock.sentinel.service,
            'test_policy',
            ', '.join(policy_names),
            "Allowed",
            "Allowed")

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_with_callable_rule(self, mock_authority,
                                                mock_log):
        """Test that a callable as the rule is evaluated correctly."""
        mock_authority.PolicyAuthority.return_value.allowed.return_value = True

        @rbac_rv.action(mock.sentinel.service,
                        rules=[lambda: mock.sentinel.action])
        def test_policy(*args):
            pass

        test_policy(self.test_obj)

        policy_authority = mock_authority.PolicyAuthority.return_value
        policy_authority.allowed.assert_called_with(
            mock.sentinel.action,
            self.test_obj.get_all_needed_roles(CONF.patrole.rbac_test_roles))

        mock_log.error.assert_not_called()

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_with_conditional_callable_rule(
            self, mock_authority, mock_log):
        """Test that a complex callable with conditional logic as the rule is
        evaluated correctly.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value = True
        expected_roles = self.test_obj.get_all_needed_roles(
            CONF.patrole.rbac_test_roles)

        def partial_func(x):
            return "foo" if x == "bar" else "qux"
        foo_callable = functools.partial(partial_func, "bar")
        bar_callable = functools.partial(partial_func, "baz")

        @rbac_rv.action(mock.sentinel.service,
                        rules=[foo_callable])
        def test_foo_policy(*args):
            pass

        @rbac_rv.action(mock.sentinel.service,
                        rules=[bar_callable])
        def test_bar_policy(*args):
            pass

        test_foo_policy(self.test_obj)
        policy_authority = mock_authority.PolicyAuthority.return_value
        policy_authority.allowed.assert_called_with(
            "foo",
            expected_roles)
        policy_authority.allowed.reset_mock()

        test_bar_policy(self.test_obj)
        policy_authority = mock_authority.PolicyAuthority.return_value
        policy_authority.allowed.assert_called_with(
            "qux",
            expected_roles)

        mock_log.error.assert_not_called()


class RBACMultiRoleRuleValidationLoggingTest(
    BaseRBACMultiRoleRuleValidationTest, RBACRuleValidationLoggingTest):
    pass


class RBACRuleValidationNegativeTest(BaseRBACRuleValidationTest):

    def setUp(self):
        super(RBACRuleValidationNegativeTest, self).setUp()
        # This behavior is tested in separate test class below.
        self.useFixture(fixtures.MockPatchObject(
            rbac_rv, '_validate_override_role_called'))

    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_invalid_service_raises_exc(self, mock_authority):
        """Test that invalid service raises the appropriate exception."""
        mock_authority.PolicyAuthority.return_value.allowed.side_effect = (
            rbac_exceptions.RbacInvalidServiceException)

        @rbac_rv.action(mock.sentinel.service, mock.sentinel.action)
        def test_policy(*args):
            pass

        self.assertRaises(rbac_exceptions.RbacInvalidServiceException,
                          test_policy, self.test_obj)


class RBACMultiRoleRuleValidationNegativeTest(
    BaseRBACMultiRoleRuleValidationTest, RBACRuleValidationNegativeTest):
    pass


class RBACRuleValidationTestMultiPolicy(BaseRBACRuleValidationTest):
    """Test suite for validating multi-policy support for the
    ``rbac_rule_validation`` decorator.
    """

    def setUp(self):
        super(RBACRuleValidationTestMultiPolicy, self).setUp()
        # This behavior is tested in separate test class below.
        self.useFixture(fixtures.MockPatchObject(
            rbac_rv, '_validate_override_role_called'))

    def _assert_policy_authority_called_with(self, rules, mock_authority):
        m_authority = mock_authority.PolicyAuthority.return_value
        m_authority.allowed.assert_has_calls([
            mock.call(
                rule,
                self.test_obj.get_all_needed_roles(
                    CONF.patrole.rbac_test_roles)
            ) for rule in rules
        ])
        m_authority.allowed.reset_mock()

    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_multi_policy_have_permission_success(
            self, mock_authority):
        """Test that when expected result is authorized and test passes that
        the overall evaluation succeeds.
        """
        mock_authority.PolicyAuthority.return_value.allowed.\
            return_value = True

        rules = [mock.sentinel.action1, mock.sentinel.action2]

        @rbac_rv.action(mock.sentinel.service, rules=rules,
                        expected_error_codes=[403, 403])
        def test_policy(*args):
            pass

        test_policy(self.test_obj)
        self._assert_policy_authority_called_with(rules, mock_authority)

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_multi_policy_overpermission_failure(
            self, mock_authority, mock_log):
        """Test that when expected result is unauthorized and test passes that
        the overall evaluation results in an RbacOverPermissionException
        getting raised.
        """

        rules = [
            mock.sentinel.action1, mock.sentinel.action2, mock.sentinel.action3
        ]
        exp_ecodes = [403, 403, 403]

        @rbac_rv.action(mock.sentinel.service, rules=rules,
                        expected_error_codes=exp_ecodes)
        def test_policy(*args):
            pass

        def _do_test(allowed_list, fail_on_action):
            mock_authority.PolicyAuthority.return_value.allowed.side_effect = (
                allowed_list)

            error_re = ".*OverPermission: .* \[%s\]$" % fail_on_action
            self.assertRaisesRegex(
                rbac_exceptions.RbacOverPermissionException, error_re,
                test_policy, self.test_obj)
            mock_log.debug.assert_any_call(
                "%s: Expecting %d to be raised for policy name: %s",
                'test_policy', 403, fail_on_action)
            self.assertRegex(mock_log.error.mock_calls[0][1][0], error_re)
            mock_log.error.reset_mock()
            self._assert_policy_authority_called_with(rules, mock_authority)

        _do_test([True, True, False], mock.sentinel.action3)
        _do_test([False, True, True], mock.sentinel.action1)
        _do_test([True, False, True], mock.sentinel.action2)

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_multi_policy_forbidden_success(
            self, mock_authority, mock_log):
        """Test that when the expected result is unauthorized and the test
        fails that the overall evaluation results in success.
        """

        rules = [
            mock.sentinel.action1, mock.sentinel.action2, mock.sentinel.action3
        ]
        exp_ecodes = [403, 403, 403]

        @rbac_rv.action(mock.sentinel.service, rules=rules,
                        expected_error_codes=exp_ecodes)
        def test_policy(*args):
            raise exceptions.Forbidden()

        def _do_test(allowed_list, fail_on_action):
            mock_authority.PolicyAuthority.return_value.allowed.\
                side_effect = allowed_list
            test_policy(self.test_obj)
            mock_log.debug.assert_called_with(
                "%s: Expecting %d to be raised for policy name: %s",
                'test_policy', 403, fail_on_action)
            mock_log.error.assert_not_called()
            self._assert_policy_authority_called_with(rules, mock_authority)

        _do_test([True, True, False], mock.sentinel.action3)
        _do_test([False, True, True], mock.sentinel.action1)
        _do_test([True, False, True], mock.sentinel.action2)

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_multi_policy_forbidden_failure(
            self, mock_authority, mock_log):
        """Test that when the expected result is authorized and the test
        fails (with a Forbidden error code) that the overall evaluation
        results in a RbacUnderPermissionException getting raised.
        """

        # NOTE: Avoid mock.sentinel here due to weird sorting with them.
        rules = ['action1', 'action2', 'action3']

        @rbac_rv.action(mock.sentinel.service, rules=rules,
                        expected_error_codes=[403, 403, 403])
        def test_policy(*args):
            raise exceptions.Forbidden()

        mock_authority.PolicyAuthority.return_value.allowed\
            .return_value = True

        error_re = ("User with roles ['member'] was not allowed to perform "
                    "the following actions: %s. Expected allowed actions: %s. "
                    "Expected disallowed actions: []." %
                    (rules, rules)).replace('[', '\[').replace(']', '\]')
        self.assertRaisesRegex(
            rbac_exceptions.RbacUnderPermissionException, error_re,
            test_policy, self.test_obj)
        self.assertRegex(mock_log.error.mock_calls[0][1][0], error_re)
        self._assert_policy_authority_called_with(rules, mock_authority)

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_multi_actions_forbidden(
            self, mock_authority, mock_log):
        """Test that when the expected result is Forbidden because
        two of the actions fail and the first action specifies 403,
        verify that the overall evaluation results in success.
        """

        rules = [
            mock.sentinel.action1, mock.sentinel.action2, mock.sentinel.action3
        ]
        exp_ecodes = [403, 403, 404]

        @rbac_rv.action(mock.sentinel.service, rules=rules,
                        expected_error_codes=exp_ecodes)
        def test_policy(*args):
            raise exceptions.Forbidden()

        def _do_test(allowed_list, fail_on_action):
            mock_authority.PolicyAuthority.return_value.allowed.\
                side_effect = allowed_list
            test_policy(self.test_obj)
            mock_log.debug.assert_called_with(
                "%s: Expecting %d to be raised for policy name: %s",
                'test_policy', 403, fail_on_action)
            mock_log.error.assert_not_called()
            self._assert_policy_authority_called_with(rules, mock_authority)

        _do_test([False, True, False], mock.sentinel.action1)
        _do_test([False, False, True], mock.sentinel.action1)

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_multi_actions_notfound(
            self, mock_authority, mock_log):
        """Test that when the expected result is not found because
        two of the actions fail and the first action specifies 404,
        verify that the overall evaluation results in success.
        """

        rules = [
            'mock.sentinel.action1', 'mock.sentinel.action2',
            'mock.sentinel.action3', 'mock.sentinel.action4'
        ]
        exp_ecodes = [403, 404, 403, 403]

        @rbac_rv.action(mock.sentinel.service, rules=rules,
                        expected_error_codes=exp_ecodes)
        def test_policy(*args):
            raise exceptions.NotFound()

        def _do_test(allowed_list, fail_on_action):
            mock_authority.PolicyAuthority.return_value.allowed.\
                side_effect = allowed_list
            test_policy(self.test_obj)
            mock_log.debug.assert_called_with(
                "%s: Expecting %d to be raised for policy name: %s",
                'test_policy', 404, fail_on_action)
            mock_log.error.assert_not_called()
            self._assert_policy_authority_called_with(rules, mock_authority)

        _do_test([True, False, False, True], 'mock.sentinel.action2')
        _do_test([True, False, True, False], 'mock.sentinel.action2')

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_multi_policy_defaults_to_correct_error_codes(
            self, mock_authority, mock_log):
        """Test omission of expected_error_codes defaults to [403] * len(rules)
        """
        mock_authority.PolicyAuthority.return_value.allowed.\
            return_value = False
        expected_log = "%s: Expecting %d to be raised for policy name: %s"

        # Validate with single rule => expected_error_codes == [403].
        rules = [mock.sentinel.action1]

        @rbac_rv.action(mock.sentinel.service, rules=rules)
        def test_policy(*args):
            raise exceptions.Forbidden()

        test_policy(self.test_obj)
        self._assert_policy_authority_called_with(rules, mock_authority)
        # Assert that 403 is expected.
        mock_calls = [x[1] for x in mock_log.debug.mock_calls]
        self.assertTrue(
            any([(expected_log, 'test_policy', 403, rules[0]) in mock_calls]))

        # Validate with multiple rules => expected_error_codes == [403, 403].
        rules = [mock.sentinel.action1, mock.sentinel.action2]

        @rbac_rv.action(mock.sentinel.service, rules=rules)
        def test_policy(*args):
            raise exceptions.Forbidden()

        test_policy(self.test_obj)
        self._assert_policy_authority_called_with(rules, mock_authority)
        # Assert that 403 is expected.
        mock_calls = [x[1] for x in mock_log.debug.mock_calls]
        self.assertTrue(
            any([(expected_log, 'test_policy', 403, rules[0]) in mock_calls]))

    def test_prepare_multi_policy_allowed_usages(self):

        def _do_test(rules, ecodes, exp_rules, exp_ecodes):
            rule_list, ec_list = rbac_rv._prepare_multi_policy(rules, ecodes)
            self.assertEqual(rule_list, exp_rules)
            self.assertEqual(ec_list, exp_ecodes)

        # Validate that expected_error_codes defaults to 403 when no values
        # are provided.
        _do_test(["rule1"], None, ["rule1"], [403])

        # Validate that `len(rules) == len(expected_error_codes)` works when
        # both == 1.
        _do_test(["rule1"], [403], ["rule1"], [403])

        # Validate that `len(rules) == len(expected_error_codes)` works when
        # both are > 1.
        _do_test(["rule1", "rule2"], [403, 404],
                 ["rule1", "rule2"], [403, 404])

        # Validate that when only a default expected_error_code argument is
        # provided, that default value and other default values (403) are
        # filled into the expected_error_codes list.
        # Example:
        #     @rbac_rv.action(service, rules=[<rule>, <rule>])
        #     def test_policy(*args):
        #        ...
        _do_test(["rule1", "rule2"], None,
                 ["rule1", "rule2"], [403, 403])

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    def test_prepare_multi_policy_disallowed_usages(self, mock_log):

        def _do_test(rules, ecodes):
            rule_list, ec_list = rbac_rv._prepare_multi_policy(rules, ecodes)

        error_re = ("The `expected_error_codes` list is not the same length"
                    " as the `rules` list.")
        # When len(rules) > 1 then len(expected_error_codes) must be same len.
        self.assertRaisesRegex(ValueError, error_re, _do_test,
                               ["rule1", "rule2"], [403])
        # When len(expected_error_codes) > 1 len(rules) must be same len.
        self.assertRaisesRegex(ValueError, error_re, _do_test, ["rule1"],
                               [403, 404])
        error_re = ("The `rules` list must be provided if using the "
                    "`expected_error_codes` list.")
        # When expected_error_codes is provided rules must be as well.
        self.assertRaisesRegex(ValueError, error_re, _do_test, None, [404])


class RBACMultiRoleRuleValidationTestMultiPolicy(
    BaseRBACMultiRoleRuleValidationTest, RBACRuleValidationTestMultiPolicy):
    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_multi_policy_forbidden_failure(
            self, mock_authority, mock_log):
        """Test that when the expected result is authorized and the test
        fails (with a Forbidden error code) that the overall evaluation
        results in a RbacUnderPermissionException getting raised.
        """

        # NOTE: Avoid mock.sentinel here due to weird sorting with them.
        rules = ['action1', 'action2', 'action3']

        @rbac_rv.action(mock.sentinel.service, rules=rules,
                        expected_error_codes=[403, 403, 403])
        def test_policy(*args):
            raise exceptions.Forbidden()

        mock_authority.PolicyAuthority.return_value.allowed\
            .return_value = True

        error_re = ("User with roles ['member', 'anotherrole'] was not "
                    "allowed to perform the following actions: %s. Expected "
                    "allowed actions: %s. Expected disallowed actions: []." %
                    (rules, rules)).replace('[', '\[').replace(']', '\]')
        self.assertRaisesRegex(
            rbac_exceptions.RbacUnderPermissionException, error_re,
            test_policy, self.test_obj)
        self.assertRegex(mock_log.error.mock_calls[0][1][0], error_re)
        self._assert_policy_authority_called_with(rules, mock_authority)


class RBACOverrideRoleValidationTest(BaseRBACRuleValidationTest):
    """Class for validating that untimely exceptions (outside
    ``override_role`` is called) result in test failures.

    This regression tests false positives caused by test exceptions matching
    the expected exception before or after the ``override_role`` context is
    called. Also tests case where ``override_role`` is never called which is
    an invalid Patrole test.

    """

    def setUp(self):
        super(RBACOverrideRoleValidationTest, self).setUp()

        self.parent_class = self.test_obj.__class__

    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_override_role_called_inside_ctx(self,
                                                             mock_authority):
        """Test success case when the expected exception is raised within the
        override_role context.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value =\
            False

        class ChildRbacTest(self.parent_class):

            @rbac_rv.action(mock.sentinel.service, rules=["fake:rule"],
                            expected_error_codes=[404])
            def test_called(self_):
                with self_.override_role():
                    raise exceptions.NotFound()

        child_test = ChildRbacTest()
        child_test.test_called()

    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_override_role_patrole_exception_ignored(
            self, mock_authority):
        """Test success case where Patrole exception is raised (which is
        valid in case of e.g. BasePatroleResponseBodyException) after
        override_role passes.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value =\
            True

        class ChildRbacTest(self.parent_class):

            @rbac_rv.action(mock.sentinel.service, rules=["fake:rule"],
                            expected_error_codes=[404])
            def test_called(self_):
                with self_.override_role():
                    pass
                # Instances of BasePatroleResponseBodyException don't count as
                # they are part of the validation work flow.
                raise rbac_exceptions.BasePatroleResponseBodyException()

        child_test = ChildRbacTest()
        self.assertRaises(rbac_exceptions.RbacUnderPermissionException,
                          child_test.test_called)

    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_override_role_called_before_ctx(self,
                                                             mock_authority):
        """Test failure case when an exception that happens before
        ``override_role`` context, even if it is the expected exception,
        raises ``RbacOverrideRoleException``.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value =\
            False

        # This behavior should work for supported (NotFound/Forbidden) and
        # miscellaneous exceptions alike.
        for exception_type in (exceptions.NotFound,
                               Exception):
            class ChildRbacTest(self.parent_class):

                @rbac_rv.action(mock.sentinel.service, rules=["fake:rule"],
                                expected_error_codes=[404])
                def test_called_before(self_):
                    raise exception_type()

            child_test = ChildRbacTest()
            test_re = ".*before.*"
            self.assertRaisesRegex(rbac_exceptions.RbacOverrideRoleException,
                                   test_re, child_test.test_called_before)

    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_override_role_called_after_ctx(self,
                                                            mock_authority):
        """Test failure case when an exception that happens before
        ``override_role`` context, even if it is the expected exception,
        raises ``RbacOverrideRoleException``.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value =\
            False

        # This behavior should work for supported (NotFound/Forbidden) and
        # miscellaneous exceptions alike.
        for exception_type in (exceptions.NotFound,
                               Exception):
            class ChildRbacTest(self.parent_class):

                @rbac_rv.action(mock.sentinel.service, rules=["fake:rule"],
                                expected_error_codes=[404])
                def test_called_after(self_):
                    with self_.override_role():
                        pass
                    # Simulates a test tearDown failure or some such.
                    raise exception_type()

            child_test = ChildRbacTest()
            test_re = ".*after.*"
            self.assertRaisesRegex(rbac_exceptions.RbacOverrideRoleException,
                                   test_re, child_test.test_called_after)

    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_override_role_never_called(self, mock_authority):
        """Test failure case where override_role is **never** called."""
        mock_authority.PolicyAuthority.return_value.allowed.return_value =\
            False

        class ChildRbacTest(self.parent_class):

            @rbac_rv.action(mock.sentinel.service, rules=["fake:rule"],
                            expected_error_codes=[404])
            def test_never_called(self_):
                pass

        child_test = ChildRbacTest()
        test_re = ".*missing required `override_role` call.*"
        self.assertRaisesRegex(rbac_exceptions.RbacOverrideRoleException,
                               test_re, child_test.test_never_called)

    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_override_role_sequential_test_calls(
            self, mock_authority):
        """Test success/failure scenarios above across sequential test calls.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value =\
            False

        class ChildRbacTest(self.parent_class):

            @rbac_rv.action(mock.sentinel.service, rules=["fake:rule1"],
                            expected_error_codes=[404])
            def test_called(self_):
                with self_.override_role():
                    raise exceptions.NotFound()

            @rbac_rv.action(mock.sentinel.service, rules=["fake:rule2"],
                            expected_error_codes=[404])
            def test_called_before(self_):
                raise exceptions.NotFound()

        test_re = ".*before.*"

        # Test case where override role is called in first test but *not* in
        # second test.
        child_test1 = ChildRbacTest()
        child_test1.test_called()
        self.assertRaisesRegex(rbac_exceptions.RbacOverrideRoleException,
                               test_re, child_test1.test_called_before)

        # Test case where override role is *not* called in first test but is
        # in second test.
        child_test2 = ChildRbacTest()
        self.assertRaisesRegex(rbac_exceptions.RbacOverrideRoleException,
                               test_re, child_test2.test_called_before)
        child_test2.test_called()



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import exceptions


class BasePatroleException(exceptions.TempestException):
    message = "An unknown RBAC exception occurred"


class BasePatroleResponseBodyException(BasePatroleException):
    message = "Response body incomplete due to RBAC authorization failure"


class RbacMissingAttributeResponseBody(BasePatroleResponseBodyException):
    """Raised when a list or show action is missing an attribute following
    RBAC authorization failure.
    """
    message = ("The response body is missing the expected %(attribute)s due "
               "to policy enforcement failure")


class RbacPartialResponseBody(BasePatroleResponseBodyException):
    """Raised when a list action only returns a subset of the available
    resources.

    For example, admin can return more resources than member for a list action.
    """
    message = ("The response body only lists a subset of the available "
               "resources due to partial policy enforcement failure. Response "
               "body: %(body)s")


class RbacEmptyResponseBody(BasePatroleResponseBodyException):
    """Raised when a list or show action is empty following RBAC authorization
    failure.
    """
    message = "The response body is empty due to policy enforcement failure."


class RbacResourceSetupFailed(BasePatroleException):
    message = "RBAC resource setup failed"


class RbacOverPermissionException(BasePatroleException):
    """Raised when the expected result is failure but the actual result is
    pass.
    """
    message = "Unauthorized action was allowed to be performed"


class RbacUnderPermissionException(BasePatroleException):
    """Raised when the expected result is pass but the actual result is
    failure.
    """
    message = "Authorized action was not allowed to be performed"


class RbacExpectedWrongException(BasePatroleException):
    """Raised when the expected exception does not match the actual exception
    raised, when both are instances of Forbidden or NotFound, indicating
    the test provides a wrong argument to `expected_error_codes`.
    """
    message = ("Expected %(expected)s to be raised but %(actual)s was raised "
               "instead. Actual exception: %(exception)s")


class RbacInvalidServiceException(BasePatroleException):
    """Raised when an invalid service is passed to ``rbac_rule_validation``
    decorator.
    """
    message = "Attempted to test an invalid service"


class RbacParsingException(BasePatroleException):
    message = "Attempted to test an invalid policy file or action"


class RbacInvalidErrorCode(BasePatroleException):
    message = "Unsupported error code passed in test"


class RbacOverrideRoleException(BasePatroleException):
    """Raised when override_role is used incorrectly or fails somehow.

    Used for safeguarding against false positives that might occur when the
    expected exception isn't raised inside the ``override_role`` context.
    Specifically, when:

    * ``override_role`` isn't called
    * an exception is raised before ``override_role`` context
    * an exception is raised after ``override_role`` context
    """
    message = "Override role failure or incorrect usage"


class RbacValidateListException(BasePatroleException):
    """Raised when override_role_and_validate_list is used incorrectly.

    Specifically, when:

    * Neither ``resource_id`` nor ``resources`` is initialized
    * Both ``resource_id`` and ``resources`` are initialized
    * The ``ctx.resources`` variable wasn't set in
        override_role_and_validate_list context.
    """
    message = "Incorrect usage of override_role_and_validate_list: %(reason)s"



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.identity import rbac_base


class IdentityGroupsV3RbacTest(rbac_base.BaseIdentityV3RbacTest):

    def _create_user_and_add_to_new_group(self):
        """Creates a user and adds to a group for test."""
        group = self.setup_test_group()
        user = self.setup_test_user()
        self.groups_client.add_group_user(group['id'], user['id'])
        return (group['id'], user['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:create_group"])
    @decorators.idempotent_id('88377f51-9074-4d64-a22f-f8931d048c9a')
    def test_create_group(self):
        with self.override_role():
            self.setup_test_group()

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:update_group"])
    @decorators.idempotent_id('790fb7be-a657-4a64-9b83-c43425cf180b')
    def test_update_group(self):
        group = self.setup_test_group()
        new_group_name = data_utils.rand_name(
            self.__class__.__name__ + '-group')

        with self.override_role():
            self.groups_client.update_group(group['id'], name=new_group_name)

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:delete_group"])
    @decorators.idempotent_id('646b52da-2a5f-486a-afb0-51fdc86a6c12')
    def test_delete_group(self):
        group = self.setup_test_group()

        with self.override_role():
            self.groups_client.delete_group(group['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:get_group"])
    @decorators.idempotent_id('d530f0ad-42b9-429b-ad05-e53ac95a040e')
    def test_show_group(self):
        group = self.setup_test_group()

        with self.override_role():
            self.groups_client.show_group(group['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:list_groups"])
    @decorators.idempotent_id('c4d0f76b-735f-4fd0-868b-0006bc420ff4')
    def test_list_groups(self):
        with self.override_role():
            self.groups_client.list_groups()

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:add_user_to_group"])
    @decorators.idempotent_id('fdd49b74-3ed3-4736-9f0e-9027a32017ac')
    def test_add_user_group(self):
        group = self.setup_test_group()
        user = self.setup_test_user()

        with self.override_role():
            self.groups_client.add_group_user(group['id'], user['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:remove_user_from_group"])
    @decorators.idempotent_id('8a60d11c-7d2b-47e5-a0f3-9ea900ca66fe')
    def test_remove_user_group(self):
        group_id, user_id = self._create_user_and_add_to_new_group()

        with self.override_role():
            self.groups_client.delete_group_user(group_id, user_id)

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:list_users_in_group"])
    @decorators.idempotent_id('b3e394a7-079e-4a0d-a4ff-9b266293d1ee')
    def test_list_user_group(self):
        group = self.setup_test_group()

        with self.override_role():
            self.groups_client.list_group_users(group['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:check_user_in_group"])
    @decorators.idempotent_id('d3603241-fd87-4a2d-94f9-f32469d1aaba')
    def test_check_user_group(self):
        group_id, user_id = self._create_user_and_add_to_new_group()

        with self.override_role():
            self.groups_client.check_group_user_existence(group_id, user_id)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_exceptions
from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base

CONF = config.CONF


class VolumeMetadataV3RbacTest(rbac_base.BaseVolumeRbacTest):

    @classmethod
    def resource_setup(cls):
        super(VolumeMetadataV3RbacTest, cls).resource_setup()
        cls.volume = cls.create_volume()
        cls.image_id = CONF.compute.image_ref

    def _add_metadata(self, volume):
        # Create metadata for the volume.
        metadata = {"key1": "value1"}
        self.volumes_client.create_volume_metadata(
            self.volume['id'], metadata)['metadata']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.volumes_client.delete_volume_metadata_item,
                        self.volume['id'], "key1")

    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume:create_volume_metadata"])
    @decorators.idempotent_id('232bbb8b-4c29-44dc-9077-b1398c20b738')
    def test_create_volume_metadata(self):
        with self.override_role():
            self._add_metadata(self.volume)

    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume:get_volume_metadata"])
    @decorators.idempotent_id('87ea37d9-23ab-47b2-a59c-16fc4d2c6dfa')
    def test_show_volume_metadata(self):
        with self.override_role():
            self.volumes_client.show_volume_metadata(
                self.volume['id'])['metadata']

    @decorators.idempotent_id('5c0f4c19-b448-4f51-9224-dad5faddc3bb')
    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume:get_volume_metadata"])
    def test_show_volume_metadata_item(self):
        self._add_metadata(self.volume)

        with self.override_role():
            self.volumes_client.show_volume_metadata_item(
                self.volume['id'], "key1")

    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume:delete_volume_metadata"])
    @decorators.idempotent_id('7498dfc1-9db2-4423-ad20-e6dcb25d1beb')
    def test_delete_volume_metadata_item(self):
        self._add_metadata(self.volume)

        with self.override_role():
            self.volumes_client.delete_volume_metadata_item(self.volume['id'],
                                                            "key1")

    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume:update_volume_metadata"])
    @decorators.idempotent_id('8ce2ff80-99ba-49ae-9bb1-7e96729ee5af')
    def test_update_volume_metadata_item(self):
        self._add_metadata(self.volume)
        updated_metadata_item = {"key1": "value1_update"}
        with self.override_role():
            self.volumes_client.update_volume_metadata_item(
                self.volume['id'], "key1", updated_metadata_item)['meta']

    @decorators.idempotent_id('a231b445-97a5-4657-b05f-245895e88da9')
    @rbac_rule_validation.action(service="cinder",
                                 rules=["volume:update_volume_metadata"])
    def test_update_volume_metadata(self):
        self._add_metadata(self.volume)
        updated_metadata = {"key1": "value1"}
        with self.override_role():
            self.volumes_client.update_volume_metadata(self.volume['id'],
                                                       updated_metadata)

    @decorators.idempotent_id('39e8f82c-f1fc-4905-bf47-177ce2f71bb9')
    @rbac_rule_validation.action(
        service="cinder",
        rules=["volume_extension:volume_image_metadata"])
    def test_list_volumes_details_image_metadata(self):
        self.volumes_client.update_volume_image_metadata(
            self.volume['id'], image_id=self.image_id)
        self.addCleanup(self.volumes_client.delete_volume_image_metadata,
                        self.volume['id'], 'image_id')

        with self.override_role():
            resp_body = self.volumes_client.list_volumes(detail=True)[
                'volumes']
        expected_attr = 'volume_image_metadata'
        if expected_attr not in resp_body[0]:
            raise rbac_exceptions.RbacMissingAttributeResponseBody(
                attribute=expected_attr)

    @decorators.idempotent_id('53f94d52-0dd5-42cf-a3a4-59b35150b3d5')
    @rbac_rule_validation.action(
        service="cinder",
        rules=["volume_extension:volume_image_metadata"])
    def test_show_volume_details_image_metadata(self):
        self.volumes_client.update_volume_image_metadata(
            self.volume['id'], image_id=self.image_id)
        self.addCleanup(self.volumes_client.delete_volume_image_metadata,
                        self.volume['id'], 'image_id')

        with self.override_role():
            resp_body = self.volumes_client.show_volume(self.volume['id'])[
                'volume']
        expected_attr = 'volume_image_metadata'
        if expected_attr not in resp_body:
            raise rbac_exceptions.RbacMissingAttributeResponseBody(
                attribute=expected_attr)

    @decorators.idempotent_id('a9d9e825-5ea3-42e6-96f3-7ac4e97b2ed0')
    @rbac_rule_validation.action(
        service="cinder",
        rules=["volume_extension:volume_image_metadata"])
    def test_update_volume_image_metadata(self):
        with self.override_role():
            self.volumes_client.update_volume_image_metadata(
                self.volume['id'], image_id=self.image_id)
        self.addCleanup(self.volumes_client.delete_volume_image_metadata,
                        self.volume['id'], 'image_id')

    @decorators.idempotent_id('a41c8eed-2051-4a25-b401-df036faacbdc')
    @rbac_rule_validation.action(
        service="cinder",
        rules=["volume_extension:volume_image_metadata"])
    def test_delete_volume_image_metadata(self):
        self.volumes_client.update_volume_image_metadata(
            self.volume['id'], image_id=self.image_id)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.volumes_client.delete_volume_image_metadata,
                        self.volume['id'], 'image_id')

        with self.override_role():
            self.volumes_client.delete_volume_image_metadata(self.volume['id'],
                                                             'image_id')



# Copyright 2018 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base


class FlavorsExtRbacTest(base.BaseNetworkExtRbacTest):

    @classmethod
    def resource_setup(cls):
        super(FlavorsExtRbacTest, cls).resource_setup()
        providers = cls.ntp_client.list_service_providers()
        if not providers["service_providers"]:
            raise cls.skipException("No service_providers available.")
        cls.service_type = providers["service_providers"][0]["service_type"]

    @decorators.idempotent_id('2632a61b-831e-4da5-82c8-a5f7d448589b')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["create_flavor"])
    def test_create_flavor(self):
        """Create flavor.

        RBAC test for the neutron "create_flavor" policy
        """
        with self.override_role():
            flavor = self.ntp_client.create_flavor(
                service_type=self.service_type)

        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.ntp_client.delete_flavor, flavor["flavor"]["id"])

    @decorators.idempotent_id('9c53164c-117d-4b44-a5cb-96f08386513f')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_flavor",
                                        "update_flavor"],
                                 expected_error_codes=[404, 403])
    def test_update_flavor(self):
        """Update flavor.

        RBAC test for the neutron "update_flavor" policy
        """
        flavor = self.ntp_client.create_flavor(service_type=self.service_type)
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.ntp_client.delete_flavor, flavor["flavor"]["id"])

        name = data_utils.rand_name(self.__class__.__name__ + '-Flavor')
        with self.override_role():
            self.ntp_client.update_flavor(flavor["flavor"]["id"], name=name)

    @decorators.idempotent_id('1de15f9e-5080-4259-ab41-e230bb312de8')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_flavor",
                                        "delete_flavor"],
                                 expected_error_codes=[404, 403])
    def test_delete_flavor(self):
        """Delete flavor.

        RBAC test for the neutron "delete_flavor" policy
        """
        flavor = self.ntp_client.create_flavor(service_type=self.service_type)
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.ntp_client.delete_flavor, flavor["flavor"]["id"])

        with self.override_role():
            self.ntp_client.delete_flavor(flavor["flavor"]["id"])

    @decorators.idempotent_id('c2baf35f-e6c1-4833-9114-aadd9b1f6aaa')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_flavor"],
                                 expected_error_codes=[404])
    def test_show_flavor(self):
        """Show flavor.

        RBAC test for the neutron "get_flavor" policy
        """
        flavor = self.ntp_client.create_flavor(service_type=self.service_type)
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.ntp_client.delete_flavor, flavor["flavor"]["id"])

        with self.override_role():
            self.ntp_client.show_flavor(flavor["flavor"]["id"])

    @decorators.idempotent_id('ab10bd5d-987e-4255-966f-947670ffd0fa')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_flavor"])
    def test_list_flavors(self):
        """List flavors.

        RBAC test for the neutron "get_flavor" policy for "list_flavors" action
        """
        flavor = self.ntp_client.create_flavor(service_type=self.service_type)
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.ntp_client.delete_flavor, flavor["flavor"]["id"])

        with self.override_role_and_validate_list(
            admin_resource_id=flavor["flavor"]["id"]
        ) as ctx:
            ctx.resources = self.ntp_client.list_flavors()['flavors']



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.image import rbac_base


class NamespacesPropertyRbacTest(rbac_base.BaseV2ImageRbacTest):

    @classmethod
    def resource_setup(cls):
        super(NamespacesPropertyRbacTest, cls).resource_setup()
        body = cls.resource_types_client.list_resource_types()
        cls.resource_name = body['resource_types'][0]['name']

    @rbac_rule_validation.action(service="glance",
                                 rules=["add_metadef_property"])
    @decorators.idempotent_id('383555ca-677b-43e9-b809-acc2b5a0176c')
    def test_add_md_properties(self):
        """Create Image Metadef Namespace Property Test

        RBAC test for the glance add_metadef_property policy
        """
        namespace = self.create_namespace()
        property_name = data_utils.rand_name(
            self.__class__.__name__ + '-test-ns-property')
        with self.override_role():
            self.namespace_properties_client.create_namespace_property(
                namespace=namespace['namespace'], type="string",
                title=property_name, name=self.resource_name)

    @rbac_rule_validation.action(service="glance",
                                 rules=["get_metadef_properties"])
    @decorators.idempotent_id('d5177611-c2b5-4000-bd9c-1987af9222ea')
    def test_get_md_properties(self):
        """List Image Metadef Namespace Properties Test

        RBAC test for the glance get_metadef_properties policy
        """
        namespace = self.create_namespace()
        with self.override_role():
            self.namespace_properties_client.list_namespace_properties(
                namespace=namespace['namespace'])

    @rbac_rule_validation.action(service="glance",
                                 rules=["get_metadef_property"])
    @decorators.idempotent_id('cfeda2af-bcab-433e-80c7-4b40c774aed5')
    def test_get_md_property(self):
        """Get Image Metadef Namespace Property Test

        RBAC test for the glance get_metadef_property policy
        """
        namespace = self.create_namespace()
        property_name = data_utils.rand_name(
            self.__class__.__name__ + '-test-ns-property')
        self.namespace_properties_client.create_namespace_property(
            namespace=namespace['namespace'], type="string",
            title=property_name, name=self.resource_name)

        with self.override_role():
            self.namespace_properties_client.show_namespace_properties(
                namespace['namespace'], self.resource_name)

    @rbac_rule_validation.action(service="glance",
                                 rules=["modify_metadef_property"])
    @decorators.idempotent_id('fdaf9363-4010-4f2f-8192-1b28f6b22e69')
    def test_modify_md_properties(self):
        """Modify Image Metadef Namespace Policy Test

        RBAC test for the glance modify_metadef_property policy
        """
        namespace = self.create_namespace()
        property_name = data_utils.rand_name(
            self.__class__.__name__ + '-test-ns-property')
        self.namespace_properties_client.create_namespace_property(
            namespace=namespace['namespace'], type="string",
            title=property_name, name=self.resource_name)

        with self.override_role():
            self.namespace_properties_client.update_namespace_properties(
                namespace['namespace'], self.resource_name, type="string",
                title=property_name, name=self.resource_name)



# Copyright 2018 AT&T Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from patrole_tempest_plugin.tests.api.volume import rbac_base
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation


class GroupTypeSpecsRbacTest(rbac_base.BaseVolumeRbacTest):
    _api_version = 3
    min_microversion = '3.11'
    max_microversion = 'latest'

    @decorators.idempotent_id('b2859734-00ad-4a22-88ee-541698e90d12')
    @rbac_rule_validation.action(
        service="cinder",
        rules=["group:group_types_specs"]
    )
    def test_group_type_specs_create(self):
        # Create new group type
        group_type = self.create_group_type()

        # Create new group type specs
        create_specs = {
            "key1": "value1",
            "key2": "value2"
        }

        with self.override_role():
            self.group_types_client. \
                create_or_update_group_type_specs(
                    group_type['id'], create_specs)['group_specs']

    @decorators.idempotent_id('469d0253-aa13-423f-8264-231ac17effbf')
    @rbac_rule_validation.action(
        service="cinder",
        rules=["group:group_types_specs"]
    )
    def test_group_type_specs_show(self):
        group_type = self.create_group_type()
        specs = {
            "key1": "value1",
            "key2": "value2"
        }
        self.group_types_client.create_or_update_group_type_specs(
            group_type['id'], specs)['group_specs']
        # Show specified item of group type specs
        with self.override_role():
            self.group_types_client.show_group_type_specs_item(
                group_type['id'], 'key2')

    @decorators.idempotent_id('2e706a4e-dec9-46bf-9426-1c5b6f3ce102')
    @rbac_rule_validation.action(
        service="cinder",
        rules=["group:group_types_specs"]
    )
    def test_group_type_specs_update(self):
        group_type = self.create_group_type()
        # Update specified item of group type specs
        update_key = 'key3'
        update_spec = {update_key: "value3-updated"}
        self.group_types_client.create_or_update_group_type_specs(
            group_type['id'], update_spec)['group_specs']
        with self.override_role():
            self.group_types_client.update_group_type_specs_item(
                group_type['id'], update_key, update_spec)

    @decorators.idempotent_id('fd5e332b-fb2c-4957-ace9-11d60ddd5472')
    @rbac_rule_validation.action(
        service="cinder",
        rules=["group:group_types_specs"]
    )
    def test_group_type_specs_list(self):
        group_type = self.create_group_type()
        with self.override_role():
            self.group_types_client.list_group_type_specs(
                group_type['id'])['group_specs']

    @decorators.idempotent_id('d9639a07-e441-4576-baf6-7ec732b16572')
    @rbac_rule_validation.action(
        service="cinder",
        rules=["group:group_types_specs"]
    )
    def test_group_type_specs_delete(self):
        group_type = self.create_group_type()
        # Delete specified item of group type specs
        delete_key = 'key1'
        specs = {'key1': 'value1'}
        self.group_types_client.create_or_update_group_type_specs(
            group_type['id'], specs)['group_specs']
        with self.override_role():
            self.group_types_client.delete_group_type_specs_item(
                group_type['id'], delete_key)



# Copyright 2018 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base


class ServiceProfileExtRbacTest(base.BaseNetworkExtRbacTest):
    @decorators.idempotent_id('6ce76efa-7400-44c1-80ec-58f79b1d89ca')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["create_service_profile"])
    def test_create_service_profile(self):
        """Create service profile

        RBAC test for the neutron "create_service_profile" policy
        """
        with self.override_role():
            self.create_service_profile()

    @decorators.idempotent_id('e4c473b7-3ae9-4a2e-8cac-848f7b01187d')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_service_profile"],
                                 expected_error_codes=[404])
    def test_show_service_profile(self):
        """Show service profile

        RBAC test for the neutron "get_service_profile" policy
        """
        profile_id = self.create_service_profile()
        with self.override_role():
            self.ntp_client.show_service_profile(profile_id)

    @decorators.idempotent_id('a3dd719d-4cd3-40cc-b4f1-5642e2717adf')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_service_profile",
                                        "update_service_profile"],
                                 expected_error_codes=[404, 403])
    def test_update_service_profile(self):
        """Update service profile

        RBAC test for the neutron "update_service_profile" policy
        """
        profile_id = self.create_service_profile()
        with self.override_role():
            self.ntp_client.update_service_profile(profile_id, enabled=False)

    @decorators.idempotent_id('926b60c2-04fe-4339-aa44-bf27121392e8')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_service_profile",
                                        "delete_service_profile"],
                                 expected_error_codes=[404, 403])
    def test_delete_service_profile(self):
        """Delete service profile

        RBAC test for the neutron "delete_service_profile" policy
        """
        profile_id = self.create_service_profile()
        with self.override_role():
            self.ntp_client.delete_service_profile(profile_id)



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg


patrole_group = cfg.OptGroup(name='patrole', title='Patrole Testing Options')


PatroleGroup = [
    cfg.StrOpt('rbac_test_role',
               default='admin',
               deprecated_for_removal=True,
               deprecated_reason="""This option is deprecated and being
replaced with ``rbac_test_roles``.
""",
               help="""The current RBAC role against which to run
Patrole tests."""),
    cfg.ListOpt('rbac_test_roles',
                help="""List of the RBAC roles against which to run
Patrole tests.""",
                default=['admin']),
    cfg.ListOpt('custom_policy_files',
                default=['/etc/%s/policy.json'],
                help="""List of the paths to search for policy files. Each
policy path assumes that the service name is included in the path once. Also
assumes Patrole is on the same host as the policy files. The paths should be
ordered by precedence, with high-priority paths before low-priority paths. All
the paths that are found to contain the service's policy file will be used and
all policy files will be merged. Allowed ``json`` or ``yaml`` formats.
"""),
    cfg.BoolOpt('test_custom_requirements',
                default=False,
                help="""
This option determines whether Patrole should run against a
``custom_requirements_file`` which defines RBAC requirements. The
purpose of setting this flag to ``True`` is to verify that RBAC policy
is in accordance to requirements. The idea is that the
``custom_requirements_file`` precisely defines what the RBAC requirements are.

Here are the possible outcomes when running the Patrole tests against
a ``custom_requirements_file``:

YAML definition: allowed
test run: allowed
test result: pass

YAML definition: allowed
test run: not allowed
test result: fail (under-permission)

YAML definition: not allowed
test run: allowed
test result: fail (over-permission)
"""),
    cfg.StrOpt('custom_requirements_file',
               help="""
File path of the YAML file that defines your RBAC requirements. This
file must be located on the same host that Patrole runs on. The YAML
file should be written as follows:

.. code-block:: yaml

    <service_foo>:
      <api_action_a>:
        - <allowed_role_1>
        - <allowed_role_2>
        - <allowed_role_3>
      <api_action_b>:
        - <allowed_role_2>
        - <allowed_role_4>
    <service_bar>:
      <api_action_c>:
        - <allowed_role_3>

Where:

service = the service that is being tested (Cinder, Nova, etc.).

api_action = the policy action that is being tested. Examples:

* volume:create
* os_compute_api:servers:start
* add_image

allowed_role = the ``oslo.policy`` role that is allowed to perform the API.
"""),
    cfg.BoolOpt('validate_deprecated_rules', default=True,
                help="""Some of the policy rules have deprecated version,
Patrole should be able to run check against default and deprecated rules,
otherwise the result of the tests may not be correct.
""")
]


patrole_log_group = cfg.OptGroup(
    name='patrole_log', title='Patrole Logging Options')


PatroleLogGroup = [
    cfg.BoolOpt('enable_reporting',
                default=False,
                help="Enables reporting on RBAC expected and actual test "
                     "results for each Patrole test"),
    cfg.StrOpt('report_log_name',
               default='patrole.log',
               help="Name of file where output from 'enable_reporting' is "
                    "logged. Note that this file is recreated on each "
                    "invocation of patrole"),
    cfg.StrOpt('report_log_path',
               default='.',
               help="Path (relative or absolute) where the output from "
                    "'enable_reporting' is logged. This is combined with "
                    "report_log_name to generate the full path."),
]


policy_feature_enabled = cfg.OptGroup(
    name='policy-feature-enabled',
    title='Feature Flags for New or Changed Policies')


PolicyFeatureEnabledGroup = [
    # TODO(felipemonteiro): The 6 feature flags below should be removed after
    # Pike is EOL.
    cfg.BoolOpt('create_port_fixed_ips_ip_address_policy',
                default=True,
                help="""Is the Neutron policy
"create_port:fixed_ips:ip_address" available in the cloud? This policy was
changed in a backwards-incompatible way."""),
    cfg.BoolOpt('update_port_fixed_ips_ip_address_policy',
                default=True,
                help="""Is the Neutron policy
"update_port:fixed_ips:ip_address" available in the cloud? This policy was
changed in a backwards-incompatible way."""),
    cfg.BoolOpt('limits_extension_used_limits_policy',
                default=True,
                help="""Is the Cinder policy
"limits_extension:used_limits" available in the cloud? This policy was
changed in a backwards-incompatible way."""),
    cfg.BoolOpt('volume_extension_volume_actions_attach_policy',
                default=True,
                help="""Is the Cinder policy
"volume_extension:volume_actions:attach" available in the cloud? This policy
was changed in a backwards-incompatible way."""),
    cfg.BoolOpt('volume_extension_volume_actions_reserve_policy',
                default=True,
                help="""Is the Cinder policy
"volume_extension:volume_actions:reserve" available in the cloud? This policy
was changed in a backwards-incompatible way."""),
    cfg.BoolOpt('volume_extension_volume_actions_unreserve_policy',
                default=True,
                help="""Is the Cinder policy
"volume_extension:volume_actions:unreserve" available in the cloud? This policy
was changed in a backwards-incompatible way."""),
    # *** Include feature flags for groups of policies below. ***
    # Best practice is to capture new policies, removed policies, renamed
    # policies in a group, per release.
    #
    # TODO(felipemonteiro): Remove these feature flags once Stein is EOL.
    cfg.BoolOpt('removed_nova_policies_stein',
                default=True,
                help="""Are the Nova API extension policies available in the
cloud (e.g. os_compute_api:os-extended-availability-zone)? These policies were
removed in Stein because Nova API extension concept was removed in Pike."""),
    cfg.BoolOpt('removed_keystone_policies_stein',
                default=True,
                help="""Are the obsolete Keystone policies available in the
cloud (e.g. identity:[create|update|get|delete]_credential)? These policies
were removed in Stein."""),
    cfg.BoolOpt('added_cinder_policies_stein',
                default=True,
                help="""Are the Cinder Stein policies available in the cloud
(e.g. [create|update|get|delete]_encryption_policy)? These policies are added
in Stein.""")
]


def list_opts():
    """Return a list of oslo.config options available.

    The purpose of this is to allow tools like the Oslo sample config file
    generator to discover the options exposed to users.
    """
    opt_list = [
        (patrole_group, PatroleGroup),
        (patrole_log_group, PatroleLogGroup),
        (policy_feature_enabled, PolicyFeatureEnabledGroup)

    ]

    return opt_list



# Copyright 2017 AT&T Corporation.
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base


class NovaAvailabilityZoneRbacTest(rbac_base.BaseV2ComputeRbacTest):

    @classmethod
    def skip_checks(cls):
        super(NovaAvailabilityZoneRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('os-availability-zone', 'compute'):
            msg = ("%s skipped as os-availability-zone not "
                   "enabled." % cls.__name__)
            raise cls.skipException(msg)

    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-availability-zone:list"])
    @decorators.idempotent_id('cd34e7ea-d26e-4fa3-a8d0-f8883726ce3d')
    def test_get_availability_zone_list_rbac(self):
        with self.override_role():
            self.availability_zone_client.list_availability_zones()

    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-availability-zone:detail"])
    @decorators.idempotent_id('2f61c191-6ece-4f21-b487-39d749e3d38e')
    def test_get_availability_zone_list_detail_rbac(self):
        with self.override_role():
            self.availability_zone_client.list_availability_zones(detail=True)



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base


class KeypairsRbacTest(rbac_base.BaseV2ComputeRbacTest):

    def _create_keypair(self):
        key_name = data_utils.rand_name(self.__class__.__name__ + '-key')
        keypair = self.keypairs_client.create_keypair(name=key_name)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.keypairs_client.delete_keypair,
                        key_name)
        return keypair

    @decorators.idempotent_id('16e0ae81-e05f-48cd-b253-cf31ab0732f0')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-keypairs:create"])
    def test_create_keypair(self):
        with self.override_role():
            self._create_keypair()

    @decorators.idempotent_id('85a5eb99-40ec-4e77-9358-bee2cdf9d7df')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-keypairs:show"])
    def test_show_keypair(self):
        kp_name = self._create_keypair()['keypair']['name']
        with self.override_role():
            self.keypairs_client.show_keypair(kp_name)

    @decorators.idempotent_id('6bff9f1c-b809-43c1-8d63-61fbd19d49d3')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-keypairs:delete"])
    def test_delete_keypair(self):
        kp_name = self._create_keypair()['keypair']['name']
        with self.override_role():
            self.keypairs_client.delete_keypair(kp_name)

    @decorators.idempotent_id('6bb31346-ff7f-4b10-978e-170ac5fcfa3e')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-keypairs:index"])
    def test_index_keypair(self):
        with self.override_role():
            self.keypairs_client.list_keypairs()



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.identity import rbac_base


class IdentityDomainsV3RbacTest(rbac_base.BaseIdentityV3RbacTest):

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:create_domain"])
    @decorators.idempotent_id('6bdaecd4-0843-4ed6-ab64-3a57ab0cd110')
    def test_create_domain(self):
        with self.override_role():
            self.setup_test_domain()

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:update_domain"])
    @decorators.idempotent_id('6bdaecd4-0843-4ed6-ab64-3a57ab0cd111')
    def test_update_domain(self):
        domain = self.setup_test_domain()
        new_domain_name = data_utils.rand_name(
            self.__class__.__name__ + '-test_update_domain')
        with self.override_role():
            self.domains_client.update_domain(domain['id'],
                                              domain=domain,
                                              name=new_domain_name)

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:delete_domain"])
    @decorators.idempotent_id('6bdaecd4-0843-4ed6-ab64-3a57ab0cd112')
    def test_delete_domain(self):
        domain = self.setup_test_domain()
        # A domain must be deactivated to be deleted
        self.domains_client.update_domain(domain['id'],
                                          domain=domain,
                                          enabled=False)
        with self.override_role():
            self.domains_client.delete_domain(domain['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:get_domain"])
    @decorators.idempotent_id('6bdaecd4-0843-4ed6-ab64-3a57ab0cd113')
    def test_show_domain(self):
        domain = self.setup_test_domain()
        with self.override_role():
            self.domains_client.show_domain(domain['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:list_domains"])
    @decorators.idempotent_id('6bdaecd4-0843-4ed6-ab64-3a57ab0cd114')
    def test_list_domains(self):
        with self.override_role():
            self.domains_client.list_domains()



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common import api_version_utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from patrole_tempest_plugin import rbac_exceptions
from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base

CONF = config.CONF


class ServerActionsRbacTest(rbac_base.BaseV2ComputeRbacTest):

    # admin credentials used for waiters which invokes a show API call
    credentials = ['primary', 'admin']

    @classmethod
    def setup_clients(cls):
        super(ServerActionsRbacTest, cls).setup_clients()
        cls.admin_servers_client = cls.os_admin.servers_client

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources(network=True, subnet=True, router=True)
        super(ServerActionsRbacTest, cls).setup_credentials()

    @classmethod
    def resource_setup(cls):
        super(ServerActionsRbacTest, cls).resource_setup()
        cls.server_id = cls.create_test_server(wait_until='ACTIVE')['id']
        cls.flavor_ref = CONF.compute.flavor_ref
        cls.flavor_ref_alt = CONF.compute.flavor_ref_alt
        cls.image_ref = CONF.compute.image_ref

    def setUp(self):
        super(ServerActionsRbacTest, self).setUp()
        try:
            waiters.wait_for_server_status(self.servers_client,
                                           self.server_id, 'ACTIVE')
        except lib_exc.NotFound:
            # If the server was found to be deleted by a previous test,
            # a new one is built
            server = self.create_test_server(wait_until='ACTIVE')
            self.__class__.server_id = server['id']
        except Exception:
            # Recreating the server in case something happened during a test
            self.__class__.server_id = self.recreate_server(self.server_id)

    def _stop_server(self):
        self.servers_client.stop_server(self.server_id)
        waiters.wait_for_server_status(
            self.admin_servers_client, self.server_id, 'SHUTOFF')

    def _resize_server(self, flavor):
        status = self.admin_servers_client. \
            show_server(self.server_id)['server']['status']
        if status == 'RESIZED':
            return
        self.servers_client.resize_server(self.server_id, flavor)
        waiters.wait_for_server_status(
            self.admin_servers_client, self.server_id, 'VERIFY_RESIZE')

    def _confirm_resize_server(self):
        status = self.admin_servers_client. \
            show_server(self.server_id)['server']['status']
        if status == 'VERIFY_RESIZE':
            self.servers_client.confirm_resize_server(self.server_id)
        waiters.wait_for_server_status(
            self.admin_servers_client, self.server_id, 'ACTIVE')

    def _shelve_server(self):
        self.servers_client.shelve_server(self.server_id)
        self.addCleanup(self._cleanup_server_actions,
                        self.servers_client.unshelve_server,
                        self.server_id)
        offload_time = CONF.compute.shelved_offload_time
        if offload_time >= 0:
            waiters.wait_for_server_status(self.admin_servers_client,
                                           self.server_id,
                                           'SHELVED_OFFLOADED',
                                           extra_timeout=offload_time)
        else:
            waiters.wait_for_server_status(self.admin_servers_client,
                                           self.server_id,
                                           'SHELVED')

    def _pause_server(self):
        self.servers_client.pause_server(self.server_id)
        self.addCleanup(self._cleanup_server_actions,
                        self.servers_client.unpause_server,
                        self.server_id)
        waiters.wait_for_server_status(
            self.admin_servers_client, self.server_id, 'PAUSED')

    def _cleanup_server_actions(self, function, server_id, **kwargs):
        server = self.servers_client.show_server(server_id)['server']
        if server['status'] != 'ACTIVE':
            function(server_id, **kwargs)

    @decorators.idempotent_id('117f4ff2-8544-437b-824f-5e41cb6640ee')
    @testtools.skipUnless(CONF.compute_feature_enabled.pause,
                          'Pause is not available.')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-pause-server:pause"])
    def test_pause_server(self):
        with self.override_role():
            self._pause_server()

    @decorators.idempotent_id('087008cf-82fa-4eeb-ae8b-32c4126456ad')
    @testtools.skipUnless(CONF.compute_feature_enabled.pause,
                          'Pause is not available.')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-pause-server:unpause"])
    def test_unpause_server(self):
        self._pause_server()
        with self.override_role():
            self.servers_client.unpause_server(self.server_id)
        waiters.wait_for_server_status(
            self.servers_client, self.server_id, 'ACTIVE')

    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:servers:stop"])
    @decorators.idempotent_id('ab4a17d2-166f-4a6d-9944-f17baa576cf2')
    def test_stop_server(self):
        with self.override_role():
            self._stop_server()

    @decorators.attr(type='slow')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:servers:start"])
    @decorators.idempotent_id('8876bfa9-4d10-406e-a335-a57e451abb12')
    def test_start_server(self):
        self._stop_server()

        with self.override_role():
            self.servers_client.start_server(self.server_id)
        waiters.wait_for_server_status(
            self.servers_client, self.server_id, 'ACTIVE')

    @decorators.attr(type='slow')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:servers:resize"])
    @decorators.idempotent_id('0546fbdd-2d8f-4ce8-ac00-f1e2129d0765')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize is not available.')
    def test_resize_server(self):
        with self.override_role():
            self._resize_server(self.flavor_ref_alt)

    @decorators.attr(type='slow')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:servers:revert_resize"])
    @decorators.idempotent_id('d41b64b8-a72d-414a-a4c5-94e1eb5e5a96')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize is not available.')
    def test_revert_resize_server(self):
        self._resize_server(self.flavor_ref_alt)

        with self.override_role():
            self.servers_client.revert_resize_server(self.server_id)
        waiters.wait_for_server_status(
            self.servers_client, self.server_id, 'ACTIVE')

    @decorators.attr(type='slow')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:servers:confirm_resize"])
    @decorators.idempotent_id('f51620cb-dfcb-4e5d-b421-2e0edaa1316e')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize is not available.')
    def test_confirm_resize_server(self):
        self._resize_server(self.flavor_ref_alt)
        self.addCleanup(self._confirm_resize_server)
        self.addCleanup(self._resize_server, self.flavor_ref)
        self.addCleanup(self._confirm_resize_server)

        with self.override_role():
            self._confirm_resize_server()

    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:servers:rebuild"])
    @decorators.idempotent_id('54b1a30b-c96c-472c-9c83-ccaf6ec7e20b')
    def test_rebuild_server(self):
        with self.override_role():
            self.servers_client.rebuild_server(self.server_id, self.image_ref)
        waiters.wait_for_server_status(
            self.servers_client, self.server_id, 'ACTIVE')

    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:servers:reboot"])
    @decorators.idempotent_id('19f27856-56e1-44f8-8615-7257f6b85cbb')
    def test_reboot_server(self):
        with self.override_role():
            self.servers_client.reboot_server(self.server_id, type='HARD')
        waiters.wait_for_server_status(
            self.servers_client, self.server_id, 'ACTIVE')

    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:servers:index"])
    @decorators.idempotent_id('631f0d86-7607-4198-8312-9da2f05464a4')
    def test_server_index(self):
        with self.override_role():
            self.servers_client.list_servers(minimal=True)

    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:servers:detail"])
    @decorators.idempotent_id('96093480-3ce5-4a8b-b569-aed870379c24')
    def test_server_detail(self):
        with self.override_role():
            self.servers_client.list_servers(detail=True)

    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:servers:detail:get_all_tenants"])
    @decorators.idempotent_id('a9e5a1c0-acfe-49a2-b2b1-fd8b19d61f71')
    def test_server_detail_all_tenants(self):
        with self.override_role():
            self.servers_client.list_servers(detail=True, all_tenants=1)

    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:servers:index:get_all_tenants"])
    @decorators.idempotent_id('4b93ba56-69e6-41f5-82c4-84a5c4c42091')
    def test_server_index_all_tenants(self):
        with self.override_role():
            self.servers_client.list_servers(minimal=True, all_tenants=1)

    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:servers:show"])
    @decorators.idempotent_id('eaaf4f51-31b5-497f-8f0f-f527e5f70b83')
    def test_show_server(self):
        with self.override_role():
            self.servers_client.show_server(self.server_id)

    @utils.services('image')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:servers:create_image"])
    @decorators.idempotent_id('ba0ac859-99f4-4055-b5e0-e0905a44d331')
    def test_create_image(self):
        with self.override_role():
            # This function will also call show image
            self.create_image_from_server(self.server_id, wait_until='ACTIVE')

    @utils.services('image', 'volume')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:servers:create_image:allow_volume_backed"])
    @decorators.idempotent_id('8b869f73-49b3-4cc4-a0ce-ef64f8e1d6f9')
    def test_create_image_from_volume_backed_server(self):
        # volume_backed=True creates a volume and create server will be
        # requested with 'block_device_mapping_v2' with necessary values for
        # this test.
        server = self.create_test_server(volume_backed=True,
                                         wait_until='ACTIVE')
        with self.override_role():
            # This function will also call show image.
            image = self.create_image_from_server(server['id'],
                                                  wait_until='ACTIVE',
                                                  wait_for_server=False)
        self.addCleanup(self.compute_images_client.wait_for_resource_deletion,
                        image['id'])
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.compute_images_client.delete_image, image['id'])

    @decorators.idempotent_id('9fdd4630-731c-4f7c-bce5-69fa3792c52a')
    @decorators.attr(type='slow')
    @testtools.skipUnless(CONF.compute_feature_enabled.snapshot,
                          'Snapshotting not available, backup not possible.')
    @utils.services('image')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-create-backup"])
    def test_create_backup(self):
        # Prioritize glance v2 over v1 for deleting/waiting for image status.
        if CONF.image_feature_enabled.api_v2:
            glance_client = self.os_primary.image_client_v2
        elif CONF.image_feature_enabled.api_v1:
            glance_client = self.os_primary.image_client
        else:
            raise lib_exc.InvalidConfiguration(
                'Either api_v1 or api_v2 must be True in '
                '[image-feature-enabled].')

        backup_name = data_utils.rand_name(self.__class__.__name__ + '-Backup')

        with self.override_role():
            resp = self.servers_client.create_backup(
                self.server_id, backup_type='daily', rotation=1,
                name=backup_name).response

        # Prior to microversion 2.45, image ID must be parsed from location
        # header. With microversion 2.45+, image_id is returned.
        if api_version_utils.compare_version_header_to_response(
                "OpenStack-API-Version", "2.45", resp, "lt"):
            image_id = resp['image_id']
        else:
            image_id = data_utils.parse_image_id(resp['location'])

        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        glance_client.delete_image, image_id)
        waiters.wait_for_image_status(glance_client, image_id, 'active')

    @decorators.attr(type='slow')
    @decorators.idempotent_id('0b70c527-af75-4bed-9ccf-4f1310a8b60f')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-shelve:shelve"])
    def test_shelve_server(self):
        with self.override_role():
            self._shelve_server()

    @decorators.attr(type='slow')
    @decorators.idempotent_id('4b6e849a-9182-49ff-9257-e97e751b475e')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-shelve:unshelve"])
    def test_unshelve_server(self):
        self._shelve_server()
        with self.override_role():
            self.servers_client.unshelve_server(self.server_id)
        waiters.wait_for_server_status(
            self.servers_client, self.server_id, 'ACTIVE')


class ServerActionsV214RbacTest(rbac_base.BaseV2ComputeRbacTest):

    min_microversion = '2.14'
    max_microversion = 'latest'

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources(network=True, subnet=True, router=True)
        super(ServerActionsV214RbacTest, cls).setup_credentials()

    @classmethod
    def resource_setup(cls):
        super(ServerActionsV214RbacTest, cls).resource_setup()
        cls.server_id = cls.create_test_server(wait_until='ACTIVE')['id']

    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-evacuate"])
    @decorators.idempotent_id('78ecef3c-faff-412a-83be-47651963eb21')
    def test_evacuate_server(self):
        fake_host_name = data_utils.rand_name(
            self.__class__.__name__ + '-fake-host')

        # NOTE(felipemonteiro): Because evacuating a server is a risky action
        # to test in the gates, a 404 is coerced using a fake host. However,
        # the policy check is done before the 404 is thrown.
        with self.override_role():
            self.assertRaisesRegex(
                lib_exc.NotFound,
                "Compute host %s not found." % fake_host_name,
                self.servers_client.evacuate_server, self.server_id,
                host=fake_host_name)


class ServerActionsV216RbacTest(rbac_base.BaseV2ComputeRbacTest):

    # This class has test case(s) that requires at least microversion 2.16.
    # See the following link for details:
    # https://developer.openstack.org/api-ref/compute/#show-server-details
    min_microversion = '2.16'
    max_microversion = 'latest'

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources(network=True, subnet=True, router=True)
        super(ServerActionsV216RbacTest, cls).setup_credentials()

    @classmethod
    def resource_setup(cls):
        super(ServerActionsV216RbacTest, cls).resource_setup()
        cls.server_id = cls.create_test_server(wait_until='ACTIVE')['id']

    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:servers:show:host_status"])
    @decorators.idempotent_id('736da575-86f8-4b2a-9902-dd37dc9a409b')
    def test_show_server_host_status(self):
        with self.override_role():
            server = self.servers_client.show_server(self.server_id)['server']

        if 'host_status' not in server:
            raise rbac_exceptions.RbacMissingAttributeResponseBody(
                attribute='host_status')



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.identity import rbac_base


class IdentityRegionsV3RbacTest(rbac_base.BaseIdentityV3RbacTest):

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:create_region"])
    @decorators.idempotent_id('6bdaecd4-0843-4ed6-ab64-3a57ab0cd119')
    def test_create_region(self):
        with self.override_role():
            self.setup_test_region()

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:update_region"])
    @decorators.idempotent_id('6bdaecd4-0843-4ed6-ab64-3a57ab0cd120')
    def test_update_region(self):
        region = self.setup_test_region()
        new_description = data_utils.rand_name(
            self.__class__.__name__ + '-test_update_region')

        with self.override_role():
            self.regions_client.update_region(region['id'],
                                              description=new_description)

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:delete_region"])
    @decorators.idempotent_id('6bdaecd4-0843-4ed6-ab64-3a57ab0cd121')
    def test_delete_region(self):
        region = self.setup_test_region()

        with self.override_role():
            self.regions_client.delete_region(region['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:get_region"])
    @decorators.idempotent_id('6bdaecd4-0843-4ed6-ab64-3a57ab0cd122')
    def test_show_region(self):
        region = self.setup_test_region()

        with self.override_role():
            self.regions_client.show_region(region['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rules=["identity:list_regions"])
    @decorators.idempotent_id('6bdaecd4-0843-4ed6-ab64-3a57ab0cd123')
    def test_list_regions(self):
        with self.override_role():
            self.regions_client.list_regions()



# Copyright 2018 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base


class AutoAllocationTopologyExtRbacTest(base.BaseNetworkExtRbacTest):

    @classmethod
    def skip_checks(cls):
        super(AutoAllocationTopologyExtRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('auto-allocated-topology',
                                          'network'):
            msg = "auto-allocated-topology extension not enabled."
            raise cls.skipException(msg)

    @decorators.idempotent_id('299CB831-F6B2-49CA-882B-E9A8E36945A2')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_auto_allocated_topology"],
                                 expected_error_codes=[404])
    def test_show_auto_allocated_topology(self):
        """Test show auto_allocated_topology.

        RBAC test for the neutron "get_auto_allocated_topology" policy
        """
        with self.override_role():
            self.ntp_client.get_auto_allocated_topology(
                tenant_id=self.os_primary.credentials.tenant_id)

    def _ensure_network_not_in_use(cls, network_id):
        ports = cls.ntp_client.list_ports(network_id=network_id)["ports"]

        # Every subnet within network should have a router interface
        expected_ports_count = len(
            cls.ntp_client.show_network(network_id)["network"]["subnets"])
        # Every network should have a single dhcp interface
        expected_ports_count += 1

        if len(ports) != expected_ports_count:
            msg = "Auto Allocated Topology in use."
            cls.skipException(msg)

    @decorators.idempotent_id('A0606AFE-065E-4C09-8E51-58EE7FBA30A2')
    @decorators.attr(type='slow')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_auto_allocated_topology",
                                        "delete_auto_allocated_topology"],
                                 expected_error_codes=[404, 403])
    def test_delete_auto_allocated_topology(self):
        """Test delete auto_allocated_topology.

        RBAC test for the neutron "delete_auto_allocated_topology" policy
        """
        tenant_id = self.os_primary.credentials.tenant_id
        net_id = self.ntp_client.get_auto_allocated_topology(
            tenant_id=tenant_id)["auto_allocated_topology"]["id"]

        self._ensure_network_not_in_use(net_id)

        with self.override_role():
            self.ntp_client.delete_auto_allocated_topology(
                tenant_id=self.os_primary.credentials.tenant_id)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import netaddr

from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from patrole_tempest_plugin import rbac_exceptions
from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base

CONF = config.CONF


class NetworksRbacTest(base.BaseNetworkRbacTest):

    @classmethod
    def resource_setup(cls):
        super(NetworksRbacTest, cls).resource_setup()
        cls.network = cls.create_network()
        cls.cidr = netaddr.IPNetwork(CONF.network.project_network_cidr)
        cls.subnet = cls.create_subnet(
            cls.network, cidr=cls.cidr, mask_bits=24, enable_dhcp=False)

    def _create_network(self,
                        router_external=None,
                        router_private=None,
                        provider_network_type=None,
                        provider_physical_network=None,
                        provider_segmentation_id=None,
                        **kwargs):
        if router_external is not None:
            kwargs['router:external'] = router_external
        if router_private is not None:
            kwargs['router:private'] = router_private
        if provider_network_type is not None:
            kwargs['provider:network_type'] = provider_network_type
        if provider_physical_network is not None:
            kwargs['provider:physical_network'] = provider_physical_network
        if provider_segmentation_id is not None:
            kwargs['provider:segmentation_id'] = provider_segmentation_id

        network_name = data_utils.rand_name(
            self.__class__.__name__ + '-Network')
        network = self.create_network(network_name=network_name, **kwargs)
        return network

    def _update_network(self,
                        net_id=None,
                        admin=None,
                        shared_network=None,
                        router_external=None,
                        router_private=None,
                        provider_network_type=None,
                        provider_physical_network=None,
                        provider_segmentation_id=None,
                        segments=None,
                        **kwargs):
        if not net_id:
            net_id = self.network['id']

        if admin is not None:
            kwargs['admin_state_up'] = admin
        if shared_network is not None:
            kwargs['shared'] = shared_network
        if router_external is not None:
            kwargs['router:external'] = router_external
        if router_private is not None:
            kwargs['router:private'] = router_private
        if provider_network_type is not None:
            kwargs['provider:network_type'] = provider_network_type
        if provider_physical_network is not None:
            kwargs['provider:physical_network'] = provider_physical_network
        if provider_segmentation_id is not None:
            kwargs['provider:segmentation_id'] = provider_segmentation_id
        if segments is not None:
            kwargs['segments'] = segments

        updated_network = self.networks_client.update_network(
            net_id, **kwargs)['network']
        return updated_network

    @rbac_rule_validation.action(service="neutron",
                                 rules=["create_network"])
    @decorators.idempotent_id('95b9baab-1ece-4e2b-89c8-8d671d974e54')
    def test_create_network(self):

        """Create Network Test

        RBAC test for the neutron create_network policy
        """
        with self.override_role():
            self._create_network()

    @rbac_rule_validation.action(service="neutron",
                                 rules=["create_network",
                                        "create_network:is_default"])
    @decorators.idempotent_id('28602661-5ac7-407e-b739-e393f619f5e3')
    def test_create_network_is_default(self):

        """Create Is Default Network Test

        RBAC test for the neutron create_network:is_default policy
        """
        try:
            with self.override_role():
                self._create_network(is_default=True)
        except lib_exc.Conflict as exc:
            # A default network might already exist
            self.assertIn('A default external network already exists',
                          str(exc))

    @rbac_rule_validation.action(service="neutron",
                                 rules=["create_network",
                                        "create_network:shared"])
    @decorators.idempotent_id('ccabf2a9-28c8-44b2-80e6-ffd65d43eef2')
    def test_create_network_shared(self):

        """Create Shared Network Test

        RBAC test for the neutron create_network:shared policy
        """
        with self.override_role():
            self._create_network(shared=True)

    @utils.requires_ext(extension='external-net', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["create_network",
                                        "create_network:router:external"])
    @decorators.idempotent_id('51adf2a7-739c-41e0-8857-3b4c460cbd24')
    def test_create_network_router_external(self):

        """Create External Router Network Test

        RBAC test for the neutron create_network:router:external policy
        """
        with self.override_role():
            self._create_network(router_external=True)

    @utils.requires_ext(extension='provider', service='network')
    @rbac_rule_validation.action(
        service="neutron",
        rules=["create_network",
               "create_network:provider:physical_network"])
    @decorators.idempotent_id('76783fed-9ff3-4499-a0d1-82d99eec364e')
    def test_create_network_provider_physical_network(self):

        """Create Network Physical Network Provider Test

        RBAC test for neutron create_network:provider:physical_network policy
        """
        try:
            with self.override_role():
                self._create_network(provider_physical_network='provider',
                                     provider_network_type='flat')
        except lib_exc.BadRequest as exc:
            # There probably won't be a physical network called 'provider', but
            # we aren't testing state of the network
            self.assertIn("Invalid input for operation: physical_network " +
                          "'provider' unknown for flat provider network.",
                          str(exc))

    @utils.requires_ext(extension='provider', service='network')
    @rbac_rule_validation.action(
        service="neutron",
        rules=["create_network",
               "create_network:provider:network_type"])
    @decorators.idempotent_id('3c42f7b8-b80c-44ef-8fa4-69ec4b1836bc')
    def test_create_network_provider_network_type(self):

        """Create Provider Network Test

        RBAC test for the neutron create_network:provider:network_type policy
        """
        with self.override_role():
            self._create_network(provider_network_type='vxlan')

    @utils.requires_ext(extension='provider', service='network')
    @rbac_rule_validation.action(
        service="neutron",
        rules=["create_network",
               "create_network:provider:segmentation_id"])
    @decorators.idempotent_id('b9decb7b-68ef-4504-b99b-41edbf7d2af5')
    def test_create_network_provider_segmentation_id(self):

        """Create Provider Network Segmentation Id Test

        RBAC test for the neutron create_network:provider:segmentation_id
        """
        with self.override_role():
            self._create_network(provider_network_type='vxlan',
                                 provider_segmentation_id=200)

    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_network", "update_network"],
                                 expected_error_codes=[404, 403])
    @decorators.idempotent_id('6485bb4e-e110-48ae-83e1-3ec8b40c3107')
    def test_update_network(self):

        """Update Network Test

        RBAC test for the neutron update_network policy
        """
        updated_name = data_utils.rand_name(
            self.__class__.__name__ + '-Network')

        with self.override_role():
            self._update_network(name=updated_name)

    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_network",
                                        "update_network",
                                        "update_network:shared"],
                                 expected_error_codes=[404, 403, 403])
    @decorators.idempotent_id('37ea3e33-47d9-49fc-9bba-1af98fbd46d6')
    def test_update_network_shared(self):

        """Update Shared Network Test

        RBAC test for the neutron update_network:shared policy
        """
        with self.override_role():
            self._update_network(shared_network=True)
        self.addCleanup(self._update_network, shared_network=False)

    @utils.requires_ext(extension='external-net', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_network",
                                        "update_network",
                                        "update_network:router:external"],
                                 expected_error_codes=[404, 403, 403])
    @decorators.idempotent_id('34884c22-499b-4960-97f1-e2ed8522a9c9')
    def test_update_network_router_external(self):

        """Update Network Router External Test

        RBAC test for the neutron update_network:router:external policy
        """
        network = self._create_network()
        with self.override_role():
            self._update_network(net_id=network['id'], router_external=True)

    @decorators.skip_because(bug='2005489', bug_type='storyboard')
    @utils.requires_ext(extension='provider', service='network')
    @rbac_rule_validation.action(
        service="neutron",
        rules=["get_network",
               "update_network",
               "update_network:provider:network_type"],
        expected_error_codes=[404, 403, 403])
    @decorators.idempotent_id('d064ef96-662b-47b6-94b7-9106dcd7ba8c')
    def test_update_network_provider_network_type(self):

        """Update Network Provider Network Type Test

        RBAC test for neutron update_network:provider:network_type policy
        """
        try:
            with self.override_role():
                self._update_network(self.network['id'],
                                     provider_network_type='vxlan')
        except lib_exc.BadRequest as exc:
            # Per the api documentation, most plugins don't support updating
            # provider attributes
            self.assertIn(
                "Plugin does not support updating provider attributes",
                str(exc))

    @decorators.skip_because(bug='2005489', bug_type='storyboard')
    @utils.requires_ext(extension='provider', service='network')
    @rbac_rule_validation.action(
        service="neutron",
        rules=["get_network",
               "update_network",
               "update_network:provider:physical_network"],
        expected_error_codes=[404, 403, 403])
    @decorators.idempotent_id('e3a55660-f75c-494e-a1b1-a8b36cc789ef')
    def test_update_network_provider_physical_network(self):

        """Update Network Provider Physical Network Test

        RBAC test for neutron update_network:provider:physical_network policy
        """
        try:
            with self.override_role():
                self._update_network(self.network['id'],
                                     provider_physical_network='provider')
        except lib_exc.BadRequest as exc:
            # Per the api documenation, most plugins don't support updating
            # provider attributes
            self.assertIn(
                "Plugin does not support updating provider attributes",
                str(exc))

    @decorators.skip_because(bug='2005489', bug_type='storyboard')
    @utils.requires_ext(extension='provider', service='network')
    @rbac_rule_validation.action(
        service="neutron",
        rules=["get_network",
               "update_network",
               "update_network:provider:segmentation_id"],
        expected_error_codes=[404, 403, 403])
    @decorators.idempotent_id('f6164228-b670-45fd-9ff9-b101930318c7')
    def test_update_network_provider_segmentation_id(self):

        """Update Network Provider Segmentation Id Test

        RBAC test for neutron update_network:provider:segmentation_id policy
        """
        try:
            with self.override_role():
                self._update_network(self.network['id'],
                                     provider_segmentation_id=400)
        except lib_exc.BadRequest as exc:
            # Per the api documenation, most plugins don't support updating
            # provider attributes
            self.assertIn(
                "Plugin does not support updating provider attributes",
                str(exc))

    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_network"],
                                 expected_error_codes=[404])
    @decorators.idempotent_id('0eb62d04-338a-4ff4-a8fa-534e52110534')
    def test_show_network(self):

        """Show Network Test

        RBAC test for the neutron get_network policy
        """
        with self.override_role():
            self.networks_client.show_network(self.network['id'])

    @utils.requires_ext(extension='external-net', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_network",
                                        "get_network:router:external"],
                                 expected_error_codes=[404, 403])
    @decorators.idempotent_id('529e4814-22e9-413f-af48-8fefcd637344')
    def test_show_network_router_external(self):

        """Show Network Router External Test

        RBAC test for the neutron get_network:router:external policy
        """
        kwargs = {'fields': 'router:external'}

        with self.override_role():
            retrieved_network = self.networks_client.show_network(
                self.network['id'], **kwargs)['network']

        if len(retrieved_network) == 0:
            raise rbac_exceptions.RbacEmptyResponseBody()

    @utils.requires_ext(extension='provider', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_network",
                                        "get_network:provider:network_type"],
                                 expected_error_codes=[404, 403])
    @decorators.idempotent_id('6521dd60-0950-458b-8491-09d3c84ac0f4')
    def test_show_network_provider_network_type(self):

        """Show Network Prodiver Network Type Test

        RBAC test for the neutron get_network:provider:network_type policy
        """
        kwargs = {'fields': 'provider:network_type'}

        with self.override_role():
            retrieved_network = self.networks_client.show_network(
                self.network['id'], **kwargs)['network']

        if len(retrieved_network) == 0:
            raise rbac_exceptions.RbacEmptyResponseBody()

    @utils.requires_ext(extension='provider', service='network')
    @rbac_rule_validation.action(
        service="neutron",
        rules=["get_network",
               "get_network:provider:physical_network"],
        expected_error_codes=[404, 403])
    @decorators.idempotent_id('c049f11a-240c-4a85-ad43-a4d3fd0a5e39')
    def test_show_network_provider_physical_network(self):

        """Show Network Provider Physical Network Test

        RBAC test for the neutron get_network:provider:physical_network policy
        """
        kwargs = {'fields': 'provider:physical_network'}

        with self.override_role():
            retrieved_network = self.networks_client.show_network(
                self.network['id'], **kwargs)['network']

        if len(retrieved_network) == 0:
            raise rbac_exceptions.RbacEmptyResponseBody()

    @utils.requires_ext(extension='provider', service='network')
    @rbac_rule_validation.action(
        service="neutron",
        rules=["get_network",
               "get_network:provider:segmentation_id"],
        expected_error_codes=[404, 403])
    @decorators.idempotent_id('38d9f085-6365-4f81-bac9-c53c294d727e')
    def test_show_network_provider_segmentation_id(self):

        """Show Network Provider Segmentation Id Test

        RBAC test for the neutron get_network:provider:segmentation_id policy
        """
        kwargs = {'fields': 'provider:segmentation_id'}

        with self.override_role():
            retrieved_network = self.networks_client.show_network(
                self.network['id'], **kwargs)['network']

        if len(retrieved_network) == 0:
            raise rbac_exceptions.RbacEmptyResponseBody()

    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_network", "delete_network"],
                                 expected_error_codes=[404, 403])
    @decorators.idempotent_id('56ca50ed-ac58-49d6-b239-ed39e7124d5c')
    def test_delete_network(self):

        """Delete Network Test

        RBAC test for the neutron delete_network policy
        """
        network = self._create_network()
        with self.override_role():
            self.networks_client.delete_network(network['id'])

    @utils.requires_ext(extension='dhcp_agent_scheduler', service='network')
    @decorators.idempotent_id('b524f19f-fbb4-4d11-a85d-03bfae17bf0e')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_dhcp-agents"])
    def test_list_dhcp_agents_on_hosting_network(self):

        """List DHCP Agents on Hosting Network Test

        RBAC test for the neutron "get_dhcp-agents" policy
        """
        with self.override_role():
            self.networks_client.list_dhcp_agents_on_hosting_network(
                self.network['id'])

    @rbac_rule_validation.action(service="neutron", rules=["get_network"])
    @decorators.idempotent_id('53d6d826-ec9a-4407-9362-b474187fae6d')
    def test_list_networks(self):
        """List Networks

        RBAC test for the neutron ``list_networks`` function and
        the ``get_network`` policy
        """

        admin_resource_id = self.network['id']
        with (self.override_role_and_validate_list(
                admin_resource_id=admin_resource_id)) as ctx:
            ctx.resources = self.networks_client.list_networks(
                id=admin_resource_id)["networks"]



# Copyright 2018 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base


class AvailabilityZoneExtRbacTest(base.BaseNetworkExtRbacTest):

    @classmethod
    def skip_checks(cls):
        super(AvailabilityZoneExtRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('availability_zone',
                                          'network'):
            msg = "network_availability_zone extension not enabled."
            raise cls.skipException(msg)

    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_availability_zone"])
    @decorators.idempotent_id('3c521be8-c32e-11e8-a611-080027758b73')
    def test_list_availability_zone_rbac(self):

        """List all available zones.

        RBAC test for the neutron ``list_availability_zones``
        function and the ``get_availability_zone`` policy
        """
        admin_resources = (self.ntp_client.list_availability_zones()
                           ["availability_zones"])
        with self.override_role_and_validate_list(
                admin_resources=admin_resources) as ctx:
            ctx.resources = (self.ntp_client.list_availability_zones()
                             ['availability_zones'])



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base


class ServiceProvidersRbacTest(base.BaseNetworkRbacTest):

    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_service_provider"])
    @decorators.idempotent_id('15f573b7-474a-4b37-8629-7fac86553ce5')
    def test_list_service_providers(self):
        with self.override_role():
            self.service_providers_client.list_service_providers()



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import collections
import copy
import glob
import os

from oslo_log import log as logging
from oslo_policy import policy
import stevedore
from tempest import clients
from tempest.common import credentials_factory as credentials
from tempest import config

from patrole_tempest_plugin.rbac_authority import RbacAuthority
from patrole_tempest_plugin import rbac_exceptions

CONF = config.CONF
LOG = logging.getLogger(__name__)


class PolicyAuthority(RbacAuthority):
    """A class that uses ``oslo.policy`` for validating RBAC."""

    def __init__(self, project_id, user_id, service, extra_target_data=None):
        """Initialization of Policy Authority class.

        Validates whether a test role can perform a policy action by querying
        ``oslo.policy`` with necessary test data.

        If a policy file does not exist, checks whether the policy file is
        registered as a namespace under "oslo.policy.policies". Nova, for
        example, doesn't use a policy file by default; its policies are
        implemented in code and registered as "nova" under
        "oslo.policy.policies".

        If the policy file is not found in either code or in a policy file,
        then an exception is raised.

        Additionally, if a custom policy file exists along with the default
        policy in code implementation, the custom policy is prioritized.

        :param uuid project_id: project_id of object performing API call
        :param uuid user_id: user_id of object performing API call
        :param string service: service of the policy file
        :param dict extra_target_data: dictionary containing additional object
            data needed by oslo.policy to validate generic checks

        Example:

        .. code-block:: python

            # Below is the default policy implementation in code, defined in
            # a service like Nova.
            test_policies = [
                policy.DocumentedRuleDefault(
                    'service:test_rule',
                    base.RULE_ADMIN_OR_OWNER,
                    "This is a description for a test policy",
                    [
                        {
                            'method': 'POST',
                            'path': '/path/to/test/resource'
                        }
                    ]),
                    'service:another_test_rule',
                    base.RULE_ADMIN_OR_OWNER,
                    "This is a description for another test policy",
                    [
                        {
                            'method': 'GET',
                            'path': '/path/to/test/resource'
                        }
                    ]),
            ]

        .. code-block:: yaml

            # Below is the custom override of the default policy in a YAML
            # policy file. Note that the default rule is "rule:admin_or_owner"
            # and the custom rule is "rule:admin_api". The `PolicyAuthority`
            # class will use the "rule:admin_api" definition for this policy
            # action.
            "service:test_rule" : "rule:admin_api"

            # Note below that no override is provided for
            # "service:another_test_rule", which means that the default policy
            # rule is used: "rule:admin_or_owner".
        """

        if extra_target_data is None:
            extra_target_data = {}

        self.service = self.validate_service(service)

        # Prioritize dynamically searching for policy files over relying on
        # deprecated service-specific policy file locations.
        if CONF.patrole.custom_policy_files:
            self.discover_policy_files()

        self.rules = self.get_rules()
        self.project_id = project_id
        self.user_id = user_id
        self.extra_target_data = extra_target_data

    @classmethod
    def validate_service(cls, service):
        """Validate whether the service passed to ``__init__`` exists."""
        service = service.lower().strip() if service else None

        # Cache the list of available services in memory to avoid needlessly
        # doing an API call every time.
        if not hasattr(cls, 'available_services'):
            admin_mgr = clients.Manager(
                credentials.get_configured_admin_credentials())
            services_client = (admin_mgr.identity_services_v3_client
                               if CONF.identity_feature_enabled.api_v3
                               else admin_mgr.identity_services_client)
            services = services_client.list_services()['services']
            cls.available_services = [s['name'] for s in services]

        if not service or service not in cls.available_services:
            LOG.debug("%s is NOT a valid service.", service)
            raise rbac_exceptions.RbacInvalidServiceException(
                "%s is NOT a valid service." % service)

        return service

    @classmethod
    def discover_policy_files(cls):
        """Dynamically discover the policy file for each service in
        ``cls.available_services``. Pick all candidate paths found
        out of the potential paths in ``[patrole] custom_policy_files``.
        """
        if not hasattr(cls, 'policy_files'):
            cls.policy_files = collections.defaultdict(list)
            for service in cls.available_services:
                for candidate_path in CONF.patrole.custom_policy_files:
                    path = candidate_path % service
                    for filename in glob.iglob(path):
                        if os.path.isfile(filename):
                            cls.policy_files[service].append(filename)

    def allowed(self, rule_name, roles):
        """Checks if a given rule in a policy is allowed with given role.

        :param string rule_name: Policy name to pass to``oslo.policy``.
        :param List[string] roles: List of roles to validate for authorization.
        :raises RbacParsingException: If ``rule_name`` does not exist in the
            cloud (in policy file or among registered in-code policy defaults).
        """
        is_admin_context = self._is_admin_context(roles)
        is_allowed = self._allowed(
            access=self._get_access_token(roles),
            apply_rule=rule_name,
            is_admin=is_admin_context)
        return is_allowed

    def _handle_deprecated_rule(self, default):
        deprecated_rule = default.deprecated_rule
        deprecated_msg = (
            'Policy "%(old_name)s":"%(old_check_str)s" was deprecated in '
            '%(release)s in favor of "%(name)s":"%(check_str)s". Reason: '
            '%(reason)s. Either ensure your deployment is ready for the new '
            'default or copy/paste the deprecated policy into your policy '
            'file and maintain it manually.' % {
                'old_name': deprecated_rule.name,
                'old_check_str': deprecated_rule.check_str,
                'release': default.deprecated_since,
                'name': default.name,
                'check_str': default.check_str,
                'reason': default.deprecated_reason
            }
        )
        LOG.warn(deprecated_msg)
        check_str = '(%s) or (%s)' % (default.check_str,
                                      deprecated_rule.check_str)
        return policy.RuleDefault(default.name, check_str)

    def get_rules(self):
        rules = policy.Rules()
        # Check whether policy file exists and attempt to read it.
        for path in self.policy_files[self.service]:
            try:
                with open(path, 'r') as fp:
                    for k, v in policy.Rules.load(fp.read()).items():
                        if k not in rules:
                            rules[k] = v
                        # If the policy name and rule are the same, no
                        # ambiguity, so no reason to warn.
                        elif str(v) != str(rules[k]):
                            msg = ("The same policy name: %s was found in "
                                   "multiple policies files for service %s. "
                                   "This can lead to policy rule ambiguity. "
                                   "Using rule: %s; Rule from file: %s")
                            LOG.warning(msg, k, self.service, rules[k], v)
            except (ValueError, IOError):
                LOG.warning("Failed to read policy file '%s' for service %s.",
                            path, self.service)

        # Check whether policy actions are defined in code. Nova and Keystone,
        # for example, define their default policy actions in code.
        mgr = stevedore.named.NamedExtensionManager(
            'oslo.policy.policies',
            names=[self.service],
            invoke_on_load=True,
            warn_on_missing_entrypoint=False)

        if mgr:
            policy_generator = {plc.name: plc.obj for plc in mgr}
            if self.service in policy_generator:
                for rule in policy_generator[self.service]:
                    if rule.name not in rules:
                        if CONF.patrole.validate_deprecated_rules:
                            # NOTE (sergey.vilgelm):
                            # The `DocumentedRuleDefault` object has no
                            # `deprecated_rule` attribute in Pike
                            if getattr(rule, 'deprecated_rule', False):
                                rule = self._handle_deprecated_rule(rule)
                        rules[rule.name] = rule.check
                    elif str(rule.check) != str(rules[rule.name]):
                        msg = ("The same policy name: %s was found in the "
                               "policies files and in the code for service "
                               "%s. This can lead to policy rule ambiguity. "
                               "Using rule: %s; Rule from code: %s")
                        LOG.warning(msg, rule.name, self.service,
                                    rules[rule.name], rule.check)

        if not rules:
            msg = (
                'Policy files for {0} service were not found among the '
                'registered in-code policies or in any of the possible policy '
                'files: {1}.'.format(
                    self.service,
                    [loc % self.service
                     for loc in CONF.patrole.custom_policy_files]))
            raise rbac_exceptions.RbacParsingException(msg)

        return rules

    def _is_admin_context(self, roles):
        """Checks whether a role has admin context.

        If context_is_admin is contained in the policy file, then checks
        whether the given role is contained in context_is_admin. If it is not
        in the policy file, then default to context_is_admin: admin.
        """
        if 'context_is_admin' in self.rules:
            return self._allowed(
                access=self._get_access_token(roles),
                apply_rule='context_is_admin')
        return CONF.identity.admin_role in roles

    def _get_access_token(self, roles):
        access_token = {
            "token": {
                "roles": [{'name': r} for r in roles],
                "project_id": self.project_id,
                "tenant_id": self.project_id,
                "user_id": self.user_id
            }
        }
        return access_token

    def _allowed(self, access, apply_rule, is_admin=False):
        """Checks if a given rule in a policy is allowed with given ``access``.

        :param dict access: Dictionary from ``_get_access_token``.
        :param string apply_rule: Rule to be checked using ``oslo.policy``.
        :param bool is_admin: Whether admin context is used.
        """
        access_data = copy.copy(access['token'])
        access_data['roles'] = [role['name'] for role in access_data['roles']]
        access_data['is_admin'] = is_admin
        # TODO(felipemonteiro): Dynamically calculate is_admin_project rather
        # than hard-coding it to True. is_admin_project cannot be determined
        # from the role, but rather from project and domain names. For more
        # information, see:
        # https://git.openstack.org/cgit/openstack/keystone/tree/keystone/token/providers/common.py?id=37ce5417418f8acbd27f3dacb70c605b0fe48301#n150
        access_data['is_admin_project'] = True

        class Object(object):
            pass
        o = Object()
        o.rules = self.rules

        target = {"project_id": access_data['project_id'],
                  "tenant_id": access_data['project_id'],
                  "network:tenant_id": access_data['project_id'],
                  "user_id": access_data['user_id']}
        if self.extra_target_data:
            target.update(self.extra_target_data)

        result = self._try_rule(apply_rule, target, access_data, o)
        return result

    def _try_rule(self, apply_rule, target, access_data, o):
        if apply_rule not in self.rules:
            message = ('Policy action "{0}" not found in policy files: '
                       '{1} or among registered policy in code defaults for '
                       '{2} service.').format(apply_rule,
                                              self.policy_files[self.service],
                                              self.service)
            LOG.debug(message)
            raise rbac_exceptions.RbacParsingException(message)
        else:
            rule = self.rules[apply_rule]
            return rule(target, access_data, o)



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base


class FlavorManageRbacTest(rbac_base.BaseV2ComputeRbacTest):

    @classmethod
    def skip_checks(cls):
        super(FlavorManageRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('OS-FLV-EXT-DATA', 'compute'):
            msg = "OS-FLV-EXT-DATA extension not enabled."
            raise cls.skipException(msg)

    @decorators.idempotent_id('a4e7faec-7a4b-4809-9856-90d5b747ca35')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-flavor-manage:create"])
    def test_create_flavor_manage(self):
        with self.override_role():
            self.create_flavor()

    @decorators.idempotent_id('782e988e-061b-4c40-896f-a77c70c2b057')
    @rbac_rule_validation.action(
        service="nova",
        rules=["os_compute_api:os-flavor-manage:delete"])
    def test_delete_flavor_manage(self):
        flavor_id = self.create_flavor()['id']

        with self.override_role():
            self.flavors_client.delete_flavor(flavor_id)
        self.flavors_client.wait_for_resource_deletion(flavor_id)



# Copyright 2018 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base


class TrunksExtRbacTest(base.BaseNetworkExtRbacTest):

    @classmethod
    def skip_checks(cls):
        super(TrunksExtRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('trunk', 'network'):
            msg = "trunk extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(TrunksExtRbacTest, cls).resource_setup()
        cls.network = cls.create_network()
        cls.port_id = cls.create_port(cls.network)["id"]

    def create_trunk(self, port_id):
        trunk = self.ntp_client.create_trunk(port_id, [])
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.ntp_client.delete_trunk, trunk["trunk"]['id'])

        return trunk

    @decorators.idempotent_id('c02618e7-bb20-1a3a-83c8-6eec2af08130')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["create_trunk"])
    def test_create_trunk(self):
        """Create trunk.

        RBAC test for the neutron "create_trunk" policy
        """
        with self.override_role():
            self.create_trunk(self.port_id)

    @decorators.idempotent_id('c02618e7-bb20-1a3a-83c8-6eec2af08131')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_trunk"],
                                 expected_error_codes=[404])
    def test_show_trunk(self):
        """Show trunk.

        RBAC test for the neutron "get_trunk" policy
        """
        trunk = self.create_trunk(self.port_id)

        with self.override_role():
            self.ntp_client.show_trunk(trunk['trunk']['id'])

    @decorators.idempotent_id('c02618e7-bb20-1a3a-83c8-6eec2af08132')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_trunk",
                                        "delete_trunk"],
                                 expected_error_codes=[404, 403])
    def test_delete_trunk(self):
        """Delete trunk.

        RBAC test for the neutron "delete_trunk" policy
        """
        trunk = self.create_trunk(self.port_id)

        with self.override_role():
            self.ntp_client.delete_trunk(trunk['trunk']['id'])

    @decorators.idempotent_id('047badd1-e4ff-40c5-9929-99ffcb8750a7')
    @rbac_rule_validation.action(service="neutron", rules=["get_trunk"])
    def test_list_trunks(self):
        """Show trunk.

        RBAC test for the neutron ``list_trunks``` function and
        the ``get_trunk`` policy
        """
        admin_resource_id = self.create_trunk(self.port_id)["trunk"]['id']
        with (self.override_role_and_validate_list(
                admin_resource_id=admin_resource_id)) as ctx:
            ctx.resources = self.ntp_client.list_trunks(
                id=admin_resource_id)["trunks"]


class TrunksSubportsExtRbacTest(base.BaseNetworkExtRbacTest):

    @classmethod
    def skip_checks(cls):
        super(TrunksSubportsExtRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('trunk', 'network'):
            msg = "trunk extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(TrunksSubportsExtRbacTest, cls).resource_setup()
        cls.network = cls.create_network()
        cls.port_id = cls.create_port(cls.network)["id"]
        cls.trunk_id = cls.ntp_client.create_trunk(
            cls.port_id, [])['trunk']['id']

        cls.addClassResourceCleanup(
            test_utils.call_and_ignore_notfound_exc,
            cls.ntp_client.delete_trunk, cls.trunk_id)

    def create_subports(self, trunk_id, port_id):
        subports = [{'port_id': port_id,
                     'segmentation_type': 'vlan',
                     'segmentation_id': 4000}]
        sub = self.ntp_client.add_subports(trunk_id, subports)
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.ntp_client.remove_subports,
            trunk_id, subports)
        return sub["sub_ports"]

    @decorators.idempotent_id('c02618e7-bb20-1a3a-83c8-6eec2af08133')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_trunk",
                                        "get_subports"],
                                 expected_error_codes=[404, 403])
    def test_get_subports(self):
        """Get subports.

        RBAC test for the neutron "get_subports" policy.

        Error 403 expected due to implementation of subports as a part of
        trunk object.
        """
        network = self.create_network()
        port = self.create_port(network)

        self.create_subports(self.trunk_id, port["id"])

        with self.override_role():
            self.ntp_client.get_subports(self.trunk_id)

    @decorators.idempotent_id('c02618e7-bb20-1a3a-83c8-6eec2af08134')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_trunk",
                                        "add_subports"],
                                 expected_error_codes=[404, 403])
    def test_add_subports(self):
        """Add subports.

        RBAC test for the neutron "add_subports" policy

        Error 403 expected due to implementation of subports as a part of
        trunk object.
        """
        network = self.create_network()
        port = self.create_port(network)

        subports = [{'port_id': port["id"],
                     'segmentation_type': 'vlan',
                     'segmentation_id': 4000}]
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.ntp_client.remove_subports,
            self.trunk_id, subports)

        with self.override_role():
            self.ntp_client.add_subports(self.trunk_id, subports)

    @decorators.idempotent_id('c02618e7-bb20-1a3a-83c8-6eec2af08135')
    @rbac_rule_validation.action(service="neutron",
                                 rules=["get_trunk",
                                        "remove_subports"],
                                 expected_error_codes=[404, 403])
    def test_remove_subports(self):
        """Remove subports.

        RBAC test for the neutron "remove_subports" policy

        Error 403 expected due to implementation of subports as a part of
        trunk object.
        """
        network = self.create_network()
        port = self.create_port(network)

        subports = self.create_subports(self.trunk_id, port["id"])

        with self.override_role():
            self.ntp_client.remove_subports(self.trunk_id, subports)
